@@ -1,7 +1,5 @@
 # pylint: disable=W0223
 
-from __future__ import absolute_import, division, print_function
-
 import os
 import sys
 import weakref
@@ -292,9 +290,9 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         # Even though we loaded the plugins in start_glue, we re-load them here
         # in case glue was started directly by initializing this class.
-        load_plugins()
+        load_plugins(require_qt_plugins=True)
 
-        self.setWindowTitle("Glue")
+        self.set_window_title()
         self.setWindowIcon(icon)
         self.setAttribute(Qt.WA_DeleteOnClose)
         self._actions = {}
@@ -525,11 +523,11 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
     def _tweak_geometry(self):
         """Maximize window by default."""
-        self._ui.main_splitter.setStretchFactor(0, 0.1)
-        self._ui.main_splitter.setStretchFactor(1, 0.9)
-        self._ui.data_plot_splitter.setStretchFactor(0, 0.25)
-        self._ui.data_plot_splitter.setStretchFactor(1, 0.5)
-        self._ui.data_plot_splitter.setStretchFactor(2, 0.25)
+        self._ui.main_splitter.setStretchFactor(0, 1)
+        self._ui.main_splitter.setStretchFactor(1, 9)
+        self._ui.data_plot_splitter.setStretchFactor(0, 1)
+        self._ui.data_plot_splitter.setStretchFactor(1, 2)
+        self._ui.data_plot_splitter.setStretchFactor(2, 1)
 
     @property
     def tab_widget(self):
@@ -1014,6 +1012,18 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
             viewer.show()
         return viewer
 
+    def set_window_title(self, detail=None):
+        """Set the window title"""
+        if detail is None:
+            title = "Glue"
+        else:
+            title = "Glue (" + detail + ")"
+        self.setWindowTitle(title)
+
+    def _on_session_changed(self, name):
+        """Call when the session is changed"""
+        self.set_window_title(name)
+
     def _choose_save_session(self, *args):
         """ Save the data collection and hub to file.
 
@@ -1044,6 +1054,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
             self.save_session(outfile,
                               include_data="including data" in file_filter,
                               absolute_paths="absolute" in file_filter)
+        self._on_session_changed(outfile)
 
     @messagebox_on_error("Failed to restore session")
     def _restore_session(self, *args):
@@ -1121,6 +1132,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         ga = Application.restore_session(path)
         if show:
             ga.start(block=False)
+        ga._on_session_changed(path)
         return ga
 
     def has_terminal(self, create_if_not=True):
@@ -1261,6 +1273,8 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
             self.close()
 
+        return app
+
     def closeEvent(self, event):
         """Emit a message to hub before closing."""
 
