@@ -1,6 +1,4 @@
-from __future__ import absolute_import, division, print_function
-
-from mock import patch
+from unittest.mock import patch
 
 from glue import _plugin_helpers as ph
 from glue.main import load_plugins
