@@ -1,9 +1,14 @@
-from __future__ import absolute_import, division, print_function
-
 import os
 import sys
 import warnings
 
+import pytest
+
+try:
+    from qtpy import PYSIDE2
+except Exception:
+    PYSIDE2 = False
+
 from glue.config import CFG_DIR as CFG_DIR_ORIG
 
 try:
@@ -13,6 +18,7 @@ except ImportError:
 else:
     OBJGRAPH_INSTALLED = True
 
+
 STDERR_ORIGINAL = sys.stderr
 
 ON_APPVEYOR = os.environ.get('APPVEYOR', 'False') == 'True'
@@ -20,10 +26,23 @@ ON_APPVEYOR = os.environ.get('APPVEYOR', 'False') == 'True'
 
 def pytest_runtest_teardown(item, nextitem):
     sys.stderr = STDERR_ORIGINAL
+    global start_dir
+    os.chdir(start_dir)
+
+
+def pytest_addoption(parser):
+    parser.addoption("--no-optional-skip", action="store_true", default=False,
+                     help="don't skip any tests with optional dependencies")
+
+
+start_dir = None
 
 
 def pytest_configure(config):
 
+    global start_dir
+    start_dir = os.path.abspath('.')
+
     os.environ['GLUE_TESTING'] = 'True'
 
     if config.getoption('no_optional_skip'):
@@ -88,3 +107,21 @@ def pytest_unconfigure(config):
 
         # Uncomment when checking for memory leaks
         # objgraph.show_most_common_types(limit=100)
+
+
+# With PySide2, tests can fail in a non-deterministic way with the following
+# error:
+#
+#   AttributeError: 'PySide2.QtGui.QStandardItem' object has no attribute 'connect'
+#
+# Until this can be properly debugged and fixed, we xfail any test that fails
+# with this exception
+
+if PYSIDE2:
+
+    def pytest_runtest_call(__multicall__):
+        try:
+            __multicall__.execute()
+        except AttributeError as exc:
+            if 'PySide2.QtGui.QStandardItem' in str(exc):
+                pytest.xfail()
