@@ -1,12 +1,9 @@
-from __future__ import absolute_import, division, print_function
-
 import os
 import warnings
 import traceback
 from functools import wraps
 
 from glue.core.data import Subset
-from glue.external.six import string_types
 from glue.core.session import Session
 from glue.core.hub import HubListener
 from glue.core import BaseData
@@ -192,7 +189,7 @@ class Application(HubListener):
         This returns the added `Data` object.
         """
 
-        if isinstance(paths, string_types):
+        if isinstance(paths, str):
             paths = [paths]
 
         datasets = []
@@ -203,8 +200,7 @@ class Application(HubListener):
             else:
                 datasets.extend(result)
 
-        self.add_datasets(self.data_collection, datasets,
-                          auto_merge=auto_merge, **kwargs)
+        self.add_datasets(datasets, auto_merge=auto_merge, **kwargs)
 
         if len(datasets) == 1:
             return datasets[0]
@@ -238,7 +234,7 @@ class Application(HubListener):
         for label, data in kwargs.items():
             datasets.extend(parse_data(data, label))
 
-        self.add_datasets(self.data_collection, datasets)
+        self.add_datasets(datasets)
 
         if links is not None:
             self.data_collection.add_link(parse_links(self.data_collection, links))
