@@ -1,13 +1,17 @@
-from __future__ import absolute_import, division, print_function
-
 import logging
 
 import numpy as np
 import pandas as pd
 
+from glue.core.coordinate_helpers import dependent_axes, pixel2world_single_axis
 from glue.utils import (shape_to_string, coerce_numeric,
                         broadcast_to, categorical_ndarray)
 
+try:
+    import dask.array as da
+    DASK_INSTALLED = True
+except ImportError:
+    DASK_INSTALLED = False
 
 __all__ = ['Component', 'DerivedComponent', 'CategoricalComponent',
            'CoordinateComponent', 'DateTimeComponent']
@@ -132,6 +136,10 @@ class Component(object):
 
         :returns: A Component (or subclass)
         """
+
+        if DASK_INSTALLED and isinstance(data, da.Array):
+            return DaskComponent(data, units=units)
+
         data = np.asarray(data)
 
         if np.issubdtype(data.dtype, np.object_):
@@ -198,7 +206,6 @@ class CoordinateComponent(Component):
     """
 
     def __init__(self, data, axis, world=False):
-        super(CoordinateComponent, self).__init__(None, None)
         self.world = world
         self._data = data
         self.axis = axis
@@ -207,6 +214,13 @@ class CoordinateComponent(Component):
     def data(self):
         return self._calculate()
 
+    @property
+    def units(self):
+        if self.world:
+            return self._data.coords.world_axis_units[self._data.ndim - 1 - self.axis]
+        else:
+            return ''
+
     def _calculate(self, view=None):
 
         if self.world:
@@ -236,7 +250,8 @@ class CoordinateComponent(Component):
             # of the pixel coordinates are the pixel coordinates themselves.
             if isinstance(view, (tuple, list)) and isinstance(view[0], np.ndarray):
                 axis = self._data.ndim - 1 - self.axis
-                return self._data.coords.pixel2world_single_axis(*view[::-1], axis=axis)
+                return pixel2world_single_axis(self._data.coords, *view[::-1],
+                                               world_axis=axis)
 
             # For 1D arrays, slice can be given as a single slice but we need
             # to wrap it in a list to make the following code work correctly,
@@ -258,7 +273,7 @@ class CoordinateComponent(Component):
                     optimize_view = True
 
             pix_coords = []
-            dep_coords = self._data.coords.dependent_axes(self.axis)
+            dep_coords = dependent_axes(self._data.coords, self.axis)
 
             final_slice = []
             final_shape = []
@@ -295,8 +310,9 @@ class CoordinateComponent(Component):
 
             # Finally we convert these to world coordinates
             axis = self._data.ndim - 1 - self.axis
-            world_coords = self._data.coords.pixel2world_single_axis(*pix_coords[::-1],
-                                                                     axis=axis)
+            world_coords = pixel2world_single_axis(self._data.coords,
+                                                   *pix_coords[::-1],
+                                                   world_axis=axis)
 
             # We get rid of any dimension for which using the view should get
             # rid of that dimension.
@@ -465,3 +481,51 @@ class DateTimeComponent(Component):
     @property
     def datetime(self):
         return True
+
+
+class DaskComponent(Component):
+    """
+    A data component powered by a dask array.
+    """
+
+    def __init__(self, data, units=None):
+        self._data = data
+        self.units = units
+
+    @property
+    def units(self):
+        return self._units
+
+    @units.setter
+    def units(self, value):
+        if value is None:
+            self._units = ''
+        else:
+            self._units = str(value)
+
+    @property
+    def data(self):
+        return self._data
+
+    @property
+    def shape(self):
+        return self._data.shape
+
+    @property
+    def ndim(self):
+        return len(self._data.shape)
+
+    def __getitem__(self, key):
+        return self._data[key].compute()
+
+    @property
+    def numeric(self):
+        return True
+
+    @property
+    def categorical(self):
+        return False
+
+    @property
+    def datetime(self):
+        return False
