@@ -1,8 +1,6 @@
 # Combo helpers independent of GUI framework - these operate on
 # SelectionCallbackProperty objects.
 
-from __future__ import absolute_import, division, print_function
-
 import weakref
 
 from glue.core import BaseData, Subset
@@ -13,8 +11,7 @@ from glue.core.message import (DataReorderComponentMessage,
                                DataCollectionDeleteMessage,
                                DataUpdateMessage,
                                DataRenameComponentMessage)
-from glue.external.echo import delay_callback, ChoiceSeparator
-from glue.external.six import string_types
+from echo import delay_callback, ChoiceSeparator
 
 __all__ = ['ComponentIDComboHelper', 'ManualDataComboHelper',
            'DataCollectionComboHelper', 'ComboHelper', 'BaseDataComboHelper']
@@ -47,7 +44,7 @@ class ComboHelper(HubListener):
     ----------
     state : :class:`~glue.core.state_objects.State`
         The state to which the selection property belongs
-    selection_property : :class:`~glue.external.echo.SelectionCallbackProperty`
+    selection_property : :class:`~echo.SelectionCallbackProperty`
         The selection property representing the combo.
     """
 
@@ -130,7 +127,7 @@ class ComponentIDComboHelper(ComboHelper):
     ----------
     state : :class:`~glue.core.state_objects.State`
         The state to which the selection property belongs
-    selection_property : :class:`~glue.external.echo.SelectionCallbackProperty`
+    selection_property : :class:`~echo.SelectionCallbackProperty`
         The selection property representing the combo.
     data_collection : :class:`~glue.core.data_collection.DataCollection`, optional
         The data collection to which the datasets belong - if specified,
@@ -141,6 +138,8 @@ class ComponentIDComboHelper(ComboHelper):
         datasets to be added/removed
     numeric : bool, optional
         Show numeric components
+    datetime : bool, optional
+        Show datetime components
     categorical : bool, optional
         Show categorical components
     pixel_coord : bool, optional
@@ -157,12 +156,12 @@ class ComponentIDComboHelper(ComboHelper):
 
     def __init__(self, state, selection_property,
                  data_collection=None, data=None,
-                 numeric=True, categorical=True,
+                 numeric=True, datetime=True, categorical=True,
                  pixel_coord=False, world_coord=False, derived=True, none=False):
 
         super(ComponentIDComboHelper, self).__init__(state, selection_property)
 
-        if isinstance(none, string_types):
+        if isinstance(none, str):
             self._none = True
             self._none_label = none
         else:
@@ -178,6 +177,7 @@ class ComponentIDComboHelper(ComboHelper):
         self.display = display_func_label
 
         self._numeric = numeric
+        self._datetime = datetime
         self._categorical = categorical
         self._pixel_coord = pixel_coord
         self._world_coord = world_coord
@@ -215,6 +215,15 @@ class ComponentIDComboHelper(ComboHelper):
         self._numeric = value
         self.refresh()
 
+    @property
+    def datetime(self):
+        return self._datetime
+
+    @datetime.setter
+    def datetime(self, value):
+        self._datetime = value
+        self.refresh()
+
     @property
     def categorical(self):
         return self._categorical
@@ -257,7 +266,7 @@ class ComponentIDComboHelper(ComboHelper):
 
     @none.setter
     def none(self, value):
-        if isinstance(value, string_types):
+        if isinstance(value, str):
             self._none = True
             self._none_label = value
         else:
@@ -304,15 +313,10 @@ class ComponentIDComboHelper(ComboHelper):
         datasets : list
             The list of :class:`~glue.core.data.Data` objects to add
         """
-
         if self._manual_data:
             raise Exception("Cannot change data in ComponentIDComboHelper "
                             "initialized from a single dataset")
-
-        try:
-            self._data.clear()
-        except AttributeError:  # PY2
-            self._data[:] = []
+        self._data.clear()
         for data in unique_data_iter(datasets):
             self.append_data(data, refresh=False)
         self.refresh()
@@ -346,7 +350,8 @@ class ComponentIDComboHelper(ComboHelper):
 
             cids = [ChoiceSeparator('Main components')]
             for cid in data.main_components:
-                if ((data.get_kind(cid) in ('numerical', 'datetime') and self.numeric) or
+                if ((data.get_kind(cid) == 'numerical' and self.numeric) or
+                        (data.get_kind(cid) == 'datetime' and self.datetime) or
                         (data.get_kind(cid) == 'categorical' and self.categorical)):
                     cids.append(cid)
             if len(cids) > 1:
@@ -406,7 +411,7 @@ class BaseDataComboHelper(ComboHelper):
     ----------
     state : :class:`~glue.core.state_objects.State`
         The state to which the selection property belongs
-    selection_property : :class:`~glue.external.echo.SelectionCallbackProperty`
+    selection_property : :class:`~echo.SelectionCallbackProperty`
         The selection property representing the combo.
     data_collection : :class:`~glue.core.data_collection.DataCollection`
         The data collection to which the datasets belong - this is needed
@@ -487,7 +492,7 @@ class ManualDataComboHelper(BaseDataComboHelper):
     ----------
     state : :class:`~glue.core.state_objects.State`
         The state to which the selection property belongs
-    selection_property : :class:`~glue.external.echo.SelectionCallbackProperty`
+    selection_property : :class:`~echo.SelectionCallbackProperty`
         The selection property representing the combo.
     data_collection : :class:`~glue.core.data_collection.DataCollection`
         The data collection to which the datasets belong - this is needed
@@ -511,11 +516,7 @@ class ManualDataComboHelper(BaseDataComboHelper):
         datasets : list
             The list of :class:`~glue.core.data.Data` objects to add
         """
-
-        try:
-            self._datasets.clear()
-        except AttributeError:  # PY2
-            self._datasets[:] = []
+        self._datasets.clear()
         for data in unique_data_iter(datasets):
             self.append_data(data, refresh=False)
         self.refresh()
@@ -565,7 +566,7 @@ class DataCollectionComboHelper(BaseDataComboHelper):
     ----------
     state : :class:`~glue.core.state_objects.State`
         The state to which the selection property belongs
-    selection_property : :class:`~glue.external.echo.SelectionCallbackProperty`
+    selection_property : :class:`~echo.SelectionCallbackProperty`
         The selection property representing the combo.
     data_collection : :class:`~glue.core.data_collection.DataCollection`
         The data collection with which to stay in sync
