@@ -22,12 +22,12 @@ Putting this together, the simplest data factory code looks like this::
         return glue.core.Data()
 """
 
-from __future__ import absolute_import, division, print_function
-
 import os
 import warnings
 
 from glue.core.contracts import contract
+from glue.core.coordinates import IdentityCoordinates
+from glue.core.component import CoordinateComponent
 from glue.core.data import Component, BaseData, Data
 from glue.config import auto_refresh, data_factory
 from glue.backends import get_timer
@@ -156,16 +156,30 @@ class LoadLog(object):
             path = os.path.abspath(self.path)
         else:
             path = os.path.relpath(self.path)
+
+        # If there are twice as many world coordinates as number of dimensions
+        # and Data.coords is set to Coordinates or None, we need to set
+        # force_coords to True to make sure that we always restore world
+        # coordinate components even if the transform is an identity transform.
+        n_coords = len([comp for comp in self.components
+                        if isinstance(comp, CoordinateComponent)])
+        if n_coords == self.components[0].ndim * 2:
+            force_coords = True
+        else:
+            force_coords = False
+
         return dict(path=path,
                     factory=context.do(self.factory),
                     kwargs=[list(self.kwargs.items())],
-                    _protocol=1)
+                    force_coords=force_coords,
+                    _protocol=2)
 
     @classmethod
     def __setgluestate__(cls, rec, context):
         fac = context.object(rec['factory'])
         kwargs = dict(*rec['kwargs'])
         kwargs['coord_first'] = rec.get('_protocol', 0) >= 1
+        kwargs['force_coords'] = rec.get('_protocol', 0) < 2 or rec.get('force_coords')
         d = load_data(rec['path'], factory=fac, **kwargs)
         return as_list(d)[0]._load_log
 
@@ -230,6 +244,7 @@ def load_data(path, factory=None, **kwargs):
     from glue.qglue import parse_data
 
     coord_first = kwargs.pop('coord_first', True)
+    force_coords = kwargs.pop('force_coords', False)
 
     def as_data_objects(ds, lbl):
         # pack other container types like astropy tables
@@ -259,6 +274,9 @@ def load_data(path, factory=None, **kwargs):
         if not isinstance(item, Data):
             continue
 
+        if item.coords is None and force_coords:
+            item.coords = IdentityCoordinates(n_dim=item.ndim)
+
         if not item.label:
             item.label = lbl
         log.log(item)  # attaches log metadata to item
@@ -290,6 +308,8 @@ def load_data(path, factory=None, **kwargs):
 def data_label(path):
     """Convert a file path into a data label, by stripping out
     slashes, file extensions, etc."""
+    if os.path.basename(path) == '':
+        path = os.path.dirname(path)
     _, fname = os.path.split(path)
     name, _ = os.path.splitext(fname)
     return name
