@@ -15,11 +15,10 @@ The LinkManager autocreates a link from D1.id['x'] to D3.id['z']
 by chaining x2y and y2z.
 """
 
-from __future__ import absolute_import, division, print_function
-
 import logging
 
-from glue.external import six
+import numpy as np
+
 from glue.core.hub import HubListener
 from glue.core.message import DataCollectionDeleteMessage, DataRemoveComponentMessage
 from glue.core.contracts import contract
@@ -29,9 +28,11 @@ from glue.core.data import Data
 from glue.core.component import DerivedComponent
 from glue.core.exceptions import IncompatibleAttribute
 from glue.core.subset import Subset
+from glue.utils import unbroadcast
+from glue.core.coordinate_helpers import dependent_axes
 
 __all__ = ['accessible_links', 'discover_links', 'find_dependents',
-           'LinkManager', 'is_equivalent_cid']
+           'LinkManager', 'is_equivalent_cid', 'pixel_cid_to_pixel_cid_matrix']
 
 
 def accessible_links(cids, links):
@@ -235,7 +236,7 @@ class LinkManager(HubListener):
         for data in data_collection:
             links = discover_links(data, self._links | self._inverse_links)
             comps = {}
-            for cid, link in six.iteritems(links):
+            for cid, link in links.items():
                 d = DerivedComponent(data, link)
                 comps[cid] = d
             data._set_externally_derivable_components(comps)
@@ -352,7 +353,7 @@ def is_convertible_to_single_pixel_cid(data, cid):
         except IncompatibleAttribute:
             return None
         if cid in data.world_component_ids:
-            if len(data.coords.dependent_axes(target_comp.axis)) == 1:
+            if len(dependent_axes(data.coords, target_comp.axis)) == 1:
                 return data.pixel_component_ids[target_comp.axis]
             else:
                 return None
@@ -399,3 +400,26 @@ def equivalent_pixel_cids(reference, target):
         else:
             return None
     return order
+
+
+def pixel_cid_to_pixel_cid_matrix(data1, data2):
+    """
+    Given two datasets, return a boolean matrix indicating which of the pixel
+    components are linked.
+
+    The returned matrix has the shape (data1.ndim, data2.ndim).
+    """
+
+    # For simplicity, and to avoid code used elsewhere, we rely on the fact
+    # that independent coordinates are broadcasted during conversions.
+
+    matrix = np.zeros((data1.ndim, data2.ndim), dtype=bool)
+
+    for idim, pix_cid in enumerate(data1.pixel_component_ids):
+        try:
+            pix_coords = unbroadcast(data2[pix_cid, (slice(2),) * data2.ndim])
+            matrix[idim] = np.array(pix_coords.shape) == 2
+        except IncompatibleAttribute:
+            pass
+
+    return matrix
