@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 import copy
 
 import numpy as np
@@ -18,7 +16,7 @@ __all__ = ['Roi', 'RectangularROI', 'CircularROI', 'PolygonalROI',
            'AbstractMplRoi', 'MplRectangularROI', 'MplCircularROI',
            'MplPolygonalROI', 'MplXRangeROI', 'MplYRangeROI',
            'XRangeROI', 'RangeROI', 'YRangeROI', 'VertexROIBase',
-           'CategoricalROI']
+           'CategoricalROI', 'EllipticalROI']
 
 PATCH_COLOR = '#FFFF00'
 SCRUBBING_KEY = 'control'
@@ -52,6 +50,11 @@ def pixel_to_data(axes, x, y):
     return axes.transData.inverted().transform(xy)
 
 
+def pixel_to_axes(axes, x, y):
+    xy = np.column_stack((np.asarray(x).ravel(), np.asarray(y).ravel()))
+    return axes.transAxes.inverted().transform(xy)
+
+
 class Roi(object):  # pragma: no cover
 
     """
@@ -272,8 +275,8 @@ class RectangularROI(Roi):
 
     @classmethod
     def __setgluestate__(cls, rec, context):
-        return cls(xmin=rec['xmin'], xmax=rec['xmax'],
-                   ymin=rec['ymin'], ymax=rec['ymax'])
+        return cls(xmin=context.object(rec['xmin']), xmax=context.object(rec['xmax']),
+                   ymin=context.object(rec['ymin']), ymax=context.object(rec['ymax']))
 
 
 class RangeROI(Roi):
@@ -355,7 +358,10 @@ class RangeROI(Roi):
 
     @classmethod
     def __setgluestate__(cls, rec, context):
-        return cls(rec['ori'], min=rec['min'], max=rec['max'])
+        if cls is XRangeROI or cls is YRangeROI:
+            return cls(min=rec['min'], max=rec['max'])
+        else:
+            return cls(rec['ori'], min=rec['min'], max=rec['max'])
 
 
 class XRangeROI(RangeROI):
@@ -449,6 +455,10 @@ class CircularROI(Roi):
     def transformed(self, xfunc=None, yfunc=None):
         return PolygonalROI(*self.to_polygon()).transformed(xfunc=xfunc, yfunc=yfunc)
 
+    def move_to(self, xdelta, ydelta):
+        self.xc += xdelta
+        self.yc += ydelta
+
     def __gluestate__(self, context):
         return dict(xc=context.do(self.xc),
                     yc=context.do(self.yc),
@@ -459,6 +469,88 @@ class CircularROI(Roi):
         return cls(xc=rec['xc'], yc=rec['yc'], radius=rec['radius'])
 
 
+class EllipticalROI(Roi):
+    """
+    A 2D elliptical region of interest.
+    """
+
+    def __init__(self, xc=None, yc=None, radius_x=None, radius_y=None, theta=None):
+        super(EllipticalROI, self).__init__()
+        if theta is not None:
+            raise NotImplementedError("Rotated ellipses are not yet supported")
+        self.xc = xc
+        self.yc = yc
+        self.radius_x = radius_x
+        self.radius_y = radius_y
+
+    def contains(self, x, y):
+        """
+        Test whether a set of (x,y) points falls within
+        the region of interest
+
+        :param x: A list of x points
+        :param y: A list of y points
+
+        *Returns*
+
+           A list of True/False values, for whether each (x,y)
+           point falls within the ROI
+
+        """
+        if not self.defined():
+            raise UndefinedROI
+
+        if not isinstance(x, np.ndarray):
+            x = np.asarray(x)
+        if not isinstance(y, np.ndarray):
+            y = np.asarray(y)
+        return (((x - self.xc) ** 2 / self.radius_x ** 2 +
+                 (y - self.yc) ** 2 / self.radius_y ** 2) < 1.)
+
+    def reset(self):
+        """
+        Reset the rectangular region.
+        """
+        self.xc = None
+        self.yc = None
+        self.radius_x = 0.
+        self.radius_y = 0.
+
+    def defined(self):
+        """ Returns True if the ROI is defined """
+        return (self.xc is not None and
+                self.yc is not None and
+                self.radius_x is not None and
+                self.radius_y is not None)
+
+    def to_polygon(self):
+        """ Returns x, y, where each is a list of points """
+        if not self.defined():
+            return [], []
+        theta = np.linspace(0, 2 * np.pi, num=20)
+        x = self.xc + self.radius_x * np.cos(theta)
+        y = self.yc + self.radius_y * np.sin(theta)
+        return x, y
+
+    def transformed(self, xfunc=None, yfunc=None):
+        return PolygonalROI(*self.to_polygon()).transformed(xfunc=xfunc, yfunc=yfunc)
+
+    def move_to(self, xdelta, ydelta):
+        self.xc += xdelta
+        self.yc += ydelta
+
+    def __gluestate__(self, context):
+        return dict(xc=context.do(self.xc),
+                    yc=context.do(self.yc),
+                    radius_x=context.do(self.radius_x),
+                    radius_y=context.do(self.radius_y))
+
+    @classmethod
+    def __setgluestate__(cls, rec, context):
+        return cls(xc=rec['xc'], yc=rec['yc'],
+                   radius_x=rec['radius_x'], radius_y=rec['radius_y'])
+
+
 class VertexROIBase(Roi):
 
     def __init__(self, vx=None, vy=None):
@@ -691,13 +783,14 @@ class AbstractMplRoi(object):
 
     _roi_cls = None
 
-    def __init__(self, axes, roi=None):
+    def __init__(self, axes, roi=None, data_space=True):
         self._axes = axes
         self._roi = roi or self._roi_cls()
         self._previous_roi = None
         self._mid_selection = False
         self._scrubbing = False
         self._background_cache = None
+        self._data_space = data_space
 
     def _draw(self):
 
@@ -813,45 +906,52 @@ class MplRectangularROI(AbstractMplRoi):
 
     _roi_cls = RectangularROI
 
-    def __init__(self, axes):
+    def __init__(self, axes, data_space=True):
 
-        super(MplRectangularROI, self).__init__(axes)
+        super(MplRectangularROI, self).__init__(axes, data_space=data_space)
 
         self._xi = None
         self._yi = None
-
         self.plot_opts = {'edgecolor': PATCH_COLOR,
                           'facecolor': PATCH_COLOR,
                           'alpha': 0.3}
 
         self._patch = Rectangle((0., 0.), 1., 1., zorder=100)
         self._patch.set_visible(False)
+        if not self._data_space:
+            self._patch.set_transform(self._axes.transAxes)
         self._axes.add_patch(self._patch)
 
     def start_selection(self, event):
-
         if event.inaxes != self._axes:
             return False
 
+        if self._data_space:
+            xval = event.xdata
+            yval = event.ydata
+        else:
+            axes_trans = self._axes.transAxes.inverted()
+            xval, yval = axes_trans.transform([event.x, event.y])
+
         if event.key == SCRUBBING_KEY:
             if not self._roi.defined():
                 return False
-            elif not self._roi.contains(event.xdata, event.ydata):
+            elif not self._roi.contains(xval, yval):
                 return False
 
         self._store_previous_roi()
         self._store_background()
 
-        self._xi = event.xdata
-        self._yi = event.ydata
+        self._xi = xval
+        self._yi = yval
 
         if event.key == SCRUBBING_KEY:
             self._scrubbing = True
             self._cx, self._cy = self._roi.center()
         else:
             self.reset()
-            self._roi.update_limits(event.xdata, event.ydata,
-                                    event.xdata, event.ydata)
+            self._roi.update_limits(self._xi, self._xi,
+                                    self._yi, self._yi)
 
         self._mid_selection = True
 
@@ -867,14 +967,21 @@ class MplRectangularROI(AbstractMplRoi):
             if not self._roi.defined():
                 return False
 
-        if self._scrubbing:
-            self._roi.move_to(self._cx + event.xdata - self._xi,
-                              self._cy + event.ydata - self._yi)
+        if self._data_space:
+            xval = event.xdata
+            yval = event.ydata
         else:
-            self._roi.update_limits(min(event.xdata, self._xi),
-                                    min(event.ydata, self._yi),
-                                    max(event.xdata, self._xi),
-                                    max(event.ydata, self._yi))
+            axes_trans = self._axes.transAxes.inverted()
+            xval, yval = axes_trans.transform([event.x, event.y])
+
+        if self._scrubbing:
+            self._roi.move_to(self._cx + xval - self._xi,
+                              self._cy + yval - self._yi)
+        else:
+            self._roi.update_limits(min(xval, self._xi),
+                                    min(yval, self._yi),
+                                    max(xval, self._xi),
+                                    max(yval, self._yi))
 
         self._sync_patch()
         self._draw()
@@ -914,18 +1021,20 @@ class MplXRangeROI(AbstractMplRoi):
 
     _roi_cls = XRangeROI
 
-    def __init__(self, axes):
+    def __init__(self, axes, data_space=True):
 
-        super(MplXRangeROI, self).__init__(axes)
+        super(MplXRangeROI, self).__init__(axes, data_space=data_space)
 
         self._xi = None
 
         self.plot_opts = {'edgecolor': PATCH_COLOR,
                           'facecolor': PATCH_COLOR,
                           'alpha': 0.3}
-
-        trans = blended_transform_factory(self._axes.transData,
-                                          self._axes.transAxes)
+        if self._data_space:
+            trans = blended_transform_factory(self._axes.transData,
+                                              self._axes.transAxes)
+        else:
+            trans = self._axes.transAxes
         self._patch = Rectangle((0., 0.), 1., 1., transform=trans, zorder=100)
         self._patch.set_visible(False)
         self._axes.add_patch(self._patch)
@@ -934,11 +1043,16 @@ class MplXRangeROI(AbstractMplRoi):
 
         if event.inaxes != self._axes:
             return False
-
+        if self._data_space:
+            x_val = event.xdata
+            y_val = event.ydata
+        else:
+            transform = self._axes.transAxes.inverted()
+            x_val, y_val = transform.transform([event.x, event.y])
         if event.key == SCRUBBING_KEY:
             if not self._roi.defined():
                 return False
-            elif not self._roi.contains(event.xdata, event.ydata):
+            elif not self._roi.contains(x_val, y_val):
                 return False
 
         self._store_previous_roi()
@@ -946,11 +1060,11 @@ class MplXRangeROI(AbstractMplRoi):
 
         if event.key == SCRUBBING_KEY:
             self._scrubbing = True
-            self._dx = event.xdata - self._roi.center()
+            self._dx = x_val - self._roi.center()
         else:
             self.reset()
-            self._roi.set_range(event.xdata, event.xdata)
-            self._xi = event.xdata
+            self._roi.set_range(x_val, x_val)
+            self._xi = x_val
 
         self._mid_selection = True
 
@@ -966,11 +1080,17 @@ class MplXRangeROI(AbstractMplRoi):
             if not self._roi.defined():
                 return False
 
-        if self._scrubbing:
-            self._roi.move_to(event.xdata + self._dx)
+        if self._data_space:
+            xval = event.xdata
         else:
-            self._roi.set_range(min(event.xdata, self._xi),
-                                max(event.xdata, self._xi))
+            axes_trans = self._axes.transAxes.inverted()
+            xval, _ = axes_trans.transform([event.x, event.y])
+
+        if self._scrubbing:
+            self._roi.move_to(xval + self._dx)
+        else:
+            self._roi.set_range(min(xval, self._xi),
+                                max(xval, self._xi))
 
         self._sync_patch()
         self._draw()
@@ -1005,18 +1125,20 @@ class MplYRangeROI(AbstractMplRoi):
 
     _roi_cls = YRangeROI
 
-    def __init__(self, axes):
+    def __init__(self, axes, data_space=True):
 
-        super(MplYRangeROI, self).__init__(axes)
+        super(MplYRangeROI, self).__init__(axes, data_space=data_space)
 
-        self._xi = None
+        self._yi = None
 
         self.plot_opts = {'edgecolor': PATCH_COLOR,
                           'facecolor': PATCH_COLOR,
                           'alpha': 0.3}
-
-        trans = blended_transform_factory(self._axes.transAxes,
-                                          self._axes.transData)
+        if self._data_space:
+            trans = blended_transform_factory(self._axes.transAxes,
+                                              self._axes.transData)
+        else:
+            trans = self._axes.transAxes
         self._patch = Rectangle((0., 0.), 1., 1., transform=trans, zorder=100)
         self._patch.set_visible(False)
         self._axes.add_patch(self._patch)
@@ -1026,10 +1148,17 @@ class MplYRangeROI(AbstractMplRoi):
         if event.inaxes != self._axes:
             return False
 
+        if self._data_space:
+            xval = event.xdata
+            yval = event.ydata
+        else:
+            axes_trans = self._axes.transAxes.inverted()
+            xval, yval = axes_trans.transform([event.x, event.y])
+
         if event.key == SCRUBBING_KEY:
             if not self._roi.defined():
                 return False
-            elif not self._roi.contains(event.xdata, event.ydata):
+            elif not self._roi.contains(xval, yval):
                 return False
 
         self._store_previous_roi()
@@ -1037,11 +1166,11 @@ class MplYRangeROI(AbstractMplRoi):
 
         if event.key == SCRUBBING_KEY:
             self._scrubbing = True
-            self._dy = event.ydata - self._roi.center()
+            self._dy = yval - self._roi.center()
         else:
             self.reset()
-            self._roi.set_range(event.ydata, event.ydata)
-            self._xi = event.ydata
+            self._roi.set_range(yval, yval)
+            self._yi = yval
 
         self._mid_selection = True
 
@@ -1057,11 +1186,17 @@ class MplYRangeROI(AbstractMplRoi):
             if not self._roi.defined():
                 return False
 
-        if self._scrubbing:
-            self._roi.move_to(event.ydata + self._dy)
+        if self._data_space:
+            yval = event.ydata
         else:
-            self._roi.set_range(min(event.ydata, self._xi),
-                                max(event.ydata, self._xi))
+            axes_trans = self._axes.transAxes.inverted()
+            _, yval = axes_trans.transform([event.x, event.y])
+
+        if self._scrubbing:
+            self._roi.move_to(yval + self._dy)
+        else:
+            self._roi.set_range(min(yval, self._yi),
+                                max(yval, self._yi))
 
         self._sync_patch()
         self._draw()
@@ -1100,9 +1235,9 @@ class MplCircularROI(AbstractMplRoi):
 
     _roi_cls = CircularROI
 
-    def __init__(self, axes):
+    def __init__(self, axes, data_space=True):
 
-        super(MplCircularROI, self).__init__(axes)
+        super(MplCircularROI, self).__init__(axes, data_space=data_space)
 
         self.plot_opts = {'edgecolor': PATCH_COLOR,
                           'facecolor': PATCH_COLOR,
@@ -1189,15 +1324,35 @@ class MplCircularROI(AbstractMplRoi):
     def roi(self):
         if not self._roi.defined():
             return PolygonalROI()
-        theta = np.linspace(0, 2 * np.pi, num=200)
+
+        # Get the circular ROI parameters in pixel units
         xy_center = self._roi.get_center()
         rad = self._roi.get_radius()
-        x = xy_center[0] + rad * np.cos(theta)
-        y = xy_center[1] + rad * np.sin(theta)
-        xy_data = pixel_to_data(self._axes, x, y)
-        vx = xy_data[:, 0].ravel().tolist()
-        vy = xy_data[:, 1].ravel().tolist()
-        result = PolygonalROI(vx, vy)
+
+        # At this point, if one of the axes is not linear, we convert to a polygon
+        if (self._axes.get_xscale() != 'linear' or self._axes.get_yscale() != 'linear') and self._data_space:
+            theta = np.linspace(0, 2 * np.pi, num=200)
+            x = xy_center[0] + rad * np.cos(theta)
+            y = xy_center[1] + rad * np.sin(theta)
+            xy_data = pixel_to_data(self._axes, x, y)
+            vx = xy_data[:, 0].ravel().tolist()
+            vy = xy_data[:, 1].ravel().tolist()
+            result = PolygonalROI(vx, vy)
+        else:
+            # We should now check if the radius in data coordinates is the same
+            # along x and y, as if so then we can return a circle, otherwise we
+            # should return an ellipse.
+            x = xy_center[0] + np.array([0, 0, rad])
+            y = xy_center[1] + np.array([0, rad, rad])
+            xy_data = pixel_to_data(self._axes, x, y) if self._data_space else pixel_to_axes(self._axes, x, y)
+            rx = xy_data[2, 0] - xy_data[0, 0]
+            ry = xy_data[1, 1] - xy_data[0, 1]
+            xc, yc = xy_data[0, :]
+            if np.allclose(rx, ry):
+                return CircularROI(xc=xc, yc=yc, radius=rx)
+            else:
+                return EllipticalROI(xc=xc, yc=yc, radius_x=rx, radius_y=ry)
+
         return result
 
     def finalize_selection(self, event):
@@ -1222,9 +1377,9 @@ class MplPolygonalROI(AbstractMplRoi):
 
     _roi_cls = PolygonalROI
 
-    def __init__(self, axes, roi=None):
+    def __init__(self, axes, roi=None, data_space=True):
 
-        super(MplPolygonalROI, self).__init__(axes, roi=roi)
+        super(MplPolygonalROI, self).__init__(axes, roi=roi, data_space=data_space)
 
         self.plot_opts = {'edgecolor': PATCH_COLOR,
                           'facecolor': PATCH_COLOR,
@@ -1232,6 +1387,8 @@ class MplPolygonalROI(AbstractMplRoi):
 
         self._patch = Polygon(np.array(list(zip([0, 1], [0, 1]))), zorder=100)
         self._patch.set_visible(False)
+        if not self._data_space:
+            self._patch.set_transform(self._axes.transAxes)
         self._axes.add_patch(self._patch)
 
     def _sync_patch(self):
@@ -1249,10 +1406,17 @@ class MplPolygonalROI(AbstractMplRoi):
         if event.inaxes != self._axes:
             return False
 
+        if self._data_space:
+            xval = event.xdata
+            yval = event.ydata
+        else:
+            axes_trans = self._axes.transAxes.inverted()
+            xval, yval = axes_trans.transform([event.x, event.y])
+
         if scrubbing or event.key == SCRUBBING_KEY:
             if not self._roi.defined():
                 return False
-            elif not self._roi.contains(event.xdata, event.ydata):
+            elif not self._roi.contains(xval, yval):
                 return False
 
         self._store_previous_roi()
@@ -1260,11 +1424,11 @@ class MplPolygonalROI(AbstractMplRoi):
 
         if scrubbing or event.key == SCRUBBING_KEY:
             self._scrubbing = True
-            self._cx = event.xdata
-            self._cy = event.ydata
+            self._cx = xval
+            self._cy = yval
         else:
             self.reset()
-            self._roi.add_point(event.xdata, event.ydata)
+            self._roi.add_point(xval, yval)
 
         self._mid_selection = True
 
@@ -1280,13 +1444,20 @@ class MplPolygonalROI(AbstractMplRoi):
             if not self._roi.defined():
                 return False
 
-        if self._scrubbing:
-            self._roi.move_to(event.xdata - self._cx,
-                              event.ydata - self._cy)
-            self._cx = event.xdata
-            self._cy = event.ydata
+        if self._data_space:
+            xval = event.xdata
+            yval = event.ydata
         else:
-            self._roi.add_point(event.xdata, event.ydata)
+            axes_trans = self._axes.transAxes.inverted()
+            xval, yval = axes_trans.transform([event.x, event.y])
+
+        if self._scrubbing:
+            self._roi.move_to(xval - self._cx,
+                              yval - self._cy)
+            self._cx = xval
+            self._cy = yval
+        else:
+            self._roi.add_point(xval, yval)
 
         self._sync_patch()
         self._draw()
