@@ -51,8 +51,6 @@ serialization protocols. Versions must be sequential integers,
 starting from 1.
 """
 
-from __future__ import absolute_import, division, print_function
-
 import os
 import json
 import uuid
@@ -67,8 +65,8 @@ from inspect import isgeneratorfunction
 import numpy as np
 from matplotlib.colors import Colormap
 from matplotlib import cm
+from astropy.wcs import WCS
 
-from glue.external import six
 from glue import core
 from glue.core.data import Data
 from glue.core.component_id import ComponentID, PixelComponentID
@@ -83,14 +81,9 @@ from glue.core.roi import Roi
 from glue.core import glue_pickle as gp
 from glue.core.subset_group import coerce_subset_groups
 from glue.utils import lookup_class
-
+from glue.config import session_patch
 
 literals = tuple([type(None), float, int, bytes, bool])
-
-if six.PY2:
-    literals += (long,)  # noqa
-
-
 literals += tuple(s for s in np.ScalarType if s not in (np.datetime64, np.timedelta64))
 
 builtin_iterables = (tuple, list, set)
@@ -108,9 +101,10 @@ if not os.path.exists(PATCH_FILE) and 'site-packages.zip' in PATCH_FILE:
     PATCH_FILE = PATCH_FILE.replace('site-packages.zip', 'glue')
 
 PATH_PATCHES = {}
-for line in open(PATCH_FILE):
-    before, after = line.strip().split(' -> ')
-    PATH_PATCHES[before.strip()] = after.strip()
+with open(PATCH_FILE) as fp:
+    for line in fp:
+        before, after = line.strip().split(' -> ')
+        PATH_PATCHES[before.strip()] = after.strip()
 
 
 def save(filename, obj):
@@ -289,7 +283,7 @@ class GlueSerializer(object):
         Return a unique name for an object, and add it to the ID registry
         if necessary.
         """
-        if isinstance(obj, six.string_types):
+        if isinstance(obj, str):
             return 'st__%s' % obj
 
         if type(obj) in literals:
@@ -333,7 +327,7 @@ class GlueSerializer(object):
         Serialize an object, but do not add it to
         the ID registry
         """
-        if isinstance(obj, six.string_types):
+        if isinstance(obj, str):
             return 'st__' + obj
 
         if type(obj) in literals:
@@ -442,6 +436,12 @@ class GlueUnSerializer(object):
         self._rec = json.loads(string) if string else json.load(fobj)
         self._callbacks = []
 
+        # apply Glue defined patches
+        apply_inplace_patches(self._rec)
+        # apply user-defined patches
+        for patcher in session_patch:
+            patcher.function(self._rec)
+
     @classmethod
     def loads(cls, string):
         return cls(string=string)
@@ -484,7 +484,7 @@ class GlueUnSerializer(object):
     @core.registry.disable
     def object(self, obj_id):
 
-        if isinstance(obj_id, six.string_types):
+        if isinstance(obj_id, str):
 
             if obj_id.startswith('st__'):  # a string literal
                 return obj_id[4:]
@@ -521,7 +521,7 @@ class GlueUnSerializer(object):
             if isgeneratorfunction(func):
                 gen, obj = obj, next(obj)  # get the partially-constructed value...
 
-            if isinstance(obj_id, six.string_types):  # ... add it to the registry ...
+            if isinstance(obj_id, str):  # ... add it to the registry ...
                 self._objs[obj_id] = obj
                 self._working.remove(obj_id)
 
@@ -535,7 +535,7 @@ class GlueUnSerializer(object):
             # obj_id from te list of IDs we are currently working on, as we
             # may want to try again (this happens when using the callbacks below)
 
-            if isinstance(obj_id, six.string_types) and obj_id in self._working:
+            if isinstance(obj_id, str) and obj_id in self._working:
                 self._working.remove(obj_id)
 
         self._try_callbacks()
@@ -586,7 +586,7 @@ def _load_tuple(rec, context):
 
 @saver(list)
 def _save_list(state, context):
-    return dict(contents=[context.do(item) for item in state])
+    return dict(contents=[context.id(item) for item in state])
 
 
 @loader(list)
@@ -614,6 +614,17 @@ def _load_slice(rec, context):
     return slice(rec['start'], rec['stop'], rec['step'])
 
 
+@saver(WCS)
+def _save_wcs(wcs, context):
+    return dict(header=wcs.to_header_string())
+
+
+@loader(WCS)
+def _load_wcs(rec, context):
+    from astropy.io import fits
+    return WCS(fits.Header.fromstring(rec['header']))
+
+
 @saver(CompositeSubsetState)
 def _save_composite_subset_state(state, context):
     return dict(state1=context.id(state.state1),
@@ -656,14 +667,16 @@ def _load_range_subset_state(rec, context):
 def _save_roi_subset_state(state, context):
     return dict(xatt=context.id(state.xatt),
                 yatt=context.id(state.yatt),
-                roi=context.id(state.roi))
+                roi=context.id(state.roi),
+                pretransform=context.id(state.pretransform))
 
 
 @loader(RoiSubsetState)
 def _load_roi_subset_state(rec, context):
     return RoiSubsetState(context.object(rec['xatt']),
                           context.object(rec['yatt']),
-                          context.object(rec['roi']))
+                          context.object(rec['roi']),
+                          context.object(rec['pretransform'] if 'pretransform' in rec else None))
 
 
 @saver(InequalitySubsetState)
@@ -698,6 +711,8 @@ def _save_style(style, context):
 @loader(VisualAttributes)
 def _load_style(rec, context):
     result = VisualAttributes()
+    if 'preferred_cmap' in result._atts:
+        result._atts.remove('preferred_cmap')
     for attr in result._atts:
         setattr(result, attr, rec[attr])
     return result
@@ -844,12 +859,16 @@ def _load_data_collection_4(rec, context):
 @saver(Data)
 def _save_data(data, context):
 
-    return dict(components=[(context.id(c),
+    state = dict(components=[(context.id(c),
                              context.id(data.get_component(c)))
-                            for c in data._components],
-                subsets=[context.id(s) for s in data.subsets],
-                label=data.label,
-                coords=context.id(data.coords))
+                             for c in data._components],
+                 subsets=[context.id(s) for s in data.subsets],
+                 label=data.label)
+
+    if data.coords is not None:
+        state['coords'] = context.id(data.coords)
+
+    return state
 
 
 @saver(Data, version=2)
@@ -861,9 +880,11 @@ def _save_data_2(data, context):
 
 @loader(Data)
 def _load_data(rec, context):
+
     label = rec['label']
     result = Data(label=label)
-    result.coords = context.object(rec['coords'])
+    if 'coords' in rec:
+        result.coords = context.object(rec['coords'])
 
     # we manually rebuild pixel/world components, so
     # we override this function. This is pretty ugly
@@ -891,10 +912,13 @@ def _load_data(rec, context):
     coord = [c for c in comps if isinstance(c[1], CoordinateComponent)]
     coord = [x[0] for x in sorted(coord, key=lambda x: x[1])]
 
-    assert len(coord) == result.ndim * 2
-
-    result._world_component_ids = coord[:len(coord) // 2]
-    result._pixel_component_ids = coord[len(coord) // 2:]
+    if getattr(result, 'coords') is not None:
+        assert len(coord) == result.ndim * 2
+        result._world_component_ids = coord[:len(coord) // 2]
+        result._pixel_component_ids = coord[len(coord) // 2:]
+    else:
+        assert len(coord) == result.ndim
+        result._pixel_component_ids = coord
 
     # We can now re-generate the coordinate links
     result._set_up_coordinate_component_links(result.ndim)
@@ -1198,3 +1222,28 @@ def _save_datetime64(dt, context):
 @loader(np.datetime64)
 def _load_datetime64(rec, context):
     return np.datetime64(rec['datetime64'])
+
+
+def apply_inplace_patches(rec):
+    """
+    Apply in-place patches to a loaded session file. Ideally this should be
+    empty, except for user patches, but we use this to fix session files that
+    need fixing to be interpretable by the current version of glue.
+    """
+
+    # The following is a patch for session files made with glue 0.15.* or
+    # earlier that were read in with a developer version of glue for part of
+    # the 0.16 development cycle, and re-saved. Essentially, if coords is set
+    # to the default identity Coordinates class, we need to make sure we
+    # always preserve the world coordinate components, and we do that by
+    # setting force_coords to True.
+    for key, value in rec.items():
+        if value['_type'] == 'glue.core.data.Data':
+            if 'coords' in value and value['coords'] is not None:
+                coords = rec[value['coords']]
+                if coords['_type'] == 'glue.core.coordinates.Coordinates':
+                    for cid, comp in value['components']:
+                        if 'log' in rec[comp]:
+                            load_log = rec[rec[comp]['log']]
+                            if 'force_coords' not in load_log:
+                                load_log['force_coords'] = True
