@@ -1,17 +1,11 @@
-from __future__ import absolute_import, division, print_function
-
-try:
-    from textwrap import indent
-except ImportError:  # PY2
-    def indent(text, prefix):
-        return ''.join([prefix + line for line in text.splitlines(True)])
+from textwrap import indent
 
 from collections import defaultdict
 
 import numpy as np
 
 from glue.core import Subset
-from glue.external.echo import (delay_callback, CallbackProperty,
+from echo import (delay_callback, CallbackProperty,
                                 HasCallbackProperties, CallbackList)
 from glue.core.state import saver, loader
 from glue.core.component_id import PixelComponentID
