@@ -1,15 +1,11 @@
-from __future__ import absolute_import, division, print_function
-
 import uuid
 import numbers
 import operator
 
 import numpy as np
 
-from glue.external import six
-from glue.external.six import PY3
 from glue.core.roi import (PolygonalROI, CategoricalROI, RangeROI, XRangeROI,
-                           YRangeROI, RectangularROI)
+                           YRangeROI, RectangularROI, CircularROI, EllipticalROI, Projected3dROI)
 from glue.core.contracts import contract
 from glue.core.util import split_component_view
 from glue.core.registry import Registry
@@ -19,10 +15,10 @@ from glue.core.decorators import memoize
 from glue.core.visual import VisualAttributes
 from glue.config import settings
 from glue.utils import (view_shape, broadcast_to, floodfill, combine_slices,
-                        polygon_line_intersections, categorical_ndarray)
+                        polygon_line_intersections, categorical_ndarray, iterate_chunks)
 
 
-__all__ = ['Subset', 'SubsetState', 'RoiSubsetState', 'CategoricalROISubsetState',
+__all__ = ['Subset', 'SubsetState', 'RoiSubsetStateNd', 'RoiSubsetState', 'CategoricalROISubsetState',
            'RangeSubsetState', 'MultiRangeSubsetState', 'CompositeSubsetState',
            'OrState', 'AndState', 'XorState', 'InvertState', 'MaskSubsetState', 'CategorySubsetState',
            'ElementSubsetState', 'InequalitySubsetState', 'combine_multiple',
@@ -346,13 +342,8 @@ class Subset(object):
         cids = self.data.pixel_component_ids
         return MaskSubsetState(m, cids)
 
-    # In Python 2 we need to do this explicitly
-    def __ne__(self, other):
-        return not self.__eq__(other)
-
-    # In Python 3, if __eq__ is defined, then __hash__ has to be re-defined
-    if PY3:
-        __hash__ = object.__hash__
+    # If __eq__ is defined, then __hash__ has to be re-defined
+    __hash__ = object.__hash__
 
     # Provide convenient access to Data methods/properties that make sense
     # here too.
@@ -364,6 +355,10 @@ class Subset(object):
     def components(self):
         return self.data.components
 
+    @property
+    def coordinate_components(self):
+        return self.data.coordinate_components
+
     @property
     def derived_components(self):
         return self.data.derived_components
@@ -474,7 +469,118 @@ class SubsetState(object):
         return XorState(self, other_state)
 
 
-class RoiSubsetState(SubsetState):
+class RoiSubsetStateNd(SubsetState):
+    """
+    A subset defined as the set of points in N dimensions that lie inside
+    a region of interest (ROI).
+
+    The dimensions are defined as numerical data attributes.
+
+    Parameters
+    ----------
+    atts : list of :class:`~glue.core.component_id.ComponentID`
+        The data attributes that define the dimensions of the region.
+    roi : :class:`~glue.core.roi.Roi`
+        The region of interest.
+    pretransform: callable, optional
+        A function that can be optionally applied to the data before checking points against the region.
+    """
+
+    def __init__(self, atts=[], roi=None, pretransform=None):
+        self._atts = atts
+        self._roi = roi
+        self._pretransform = pretransform
+
+    @property
+    def roi(self):
+        """
+        The region of interest.
+        """
+        return self._roi
+
+    @roi.setter
+    def roi(self, value):
+        self._roi = value
+
+    @property
+    def pretransform(self):
+        """
+        An optional transformation function to apply before checking if points are in the ROI.
+        """
+        return self._pretransform
+
+    @pretransform.setter
+    def pretransform(self, value):
+        if not callable(value) and value is not None:
+            raise TypeError("The pretransform must be callable or None.")
+        self._pretransform = value
+
+    @property
+    def attributes(self):
+        return tuple(self._atts)
+
+    @contract(data='isinstance(Data)', view='array_view')
+    def to_mask(self, data, view=None):
+
+        # TODO: make sure that pixel components don't actually take up much
+        #       memory and are just views
+
+        raw_comps = []
+        for att in self._atts:
+            raw_comps.append(data[att, view])
+        res_shape = raw_comps[0].shape
+        if not self.roi.defined():
+            return np.zeros(raw_comps[0].shape, dtype=bool)
+
+        if raw_comps[0].ndim == data.ndim and all([att in data.pixel_component_ids for att in self._atts]):
+            # This is a special case - the ROI is defined in pixel space, so we
+            # can apply it to a single slice and then broadcast it to all other
+            # dimensions. We start off by extracting a slice which takes only
+            # the first elements of all dimensions except the attributes in
+            # question, for which we take all the elements. We need to preserve
+            # the dimensionality of the array, hence the use of slice(0, 1).
+            # Note that we can only do this if the view (if present) preserved
+            # the dimensionality, which is why we checked that raw_comps[0].ndim == data.ndim.
+            axis_ids = [att.axis for att in self._atts]
+            subset = []
+            for i in range(data.ndim):
+                if i in axis_ids:
+                    subset.append(slice(None))
+                else:
+                    subset.append(slice(0, 1))
+            for i in range(len(raw_comps)):
+                raw_comps[i] = raw_comps[i][tuple(subset)]
+
+        if self.pretransform:
+            transformed_points = []
+            for slices in iterate_chunks(raw_comps[0].shape, n_max=1000000):
+                comp_subsets = []
+                for raw_comp in raw_comps:
+                    comp_subsets.append(raw_comp[slices])
+                res = self.pretransform(*comp_subsets)
+
+                # Do this here in case the pretransform changes the dimensionality
+                # e.g. 3D input to a 2D projection like Projected3dROI does internally
+                while len(transformed_points) < len(res):
+                    transformed_points.append(np.zeros(raw_comps[0].shape))
+
+                for i in range(len(res)):
+                    transformed_points[i][slices] = res[i]
+        else:
+            transformed_points = raw_comps
+
+        if isinstance(self.roi, Projected3dROI):
+            result = self.roi.contains3d(*transformed_points)
+        else:
+            result = self.roi.contains(*transformed_points)
+
+        if result.shape != res_shape:
+            result = np.broadcast_to(result, res_shape)
+
+        return result
+
+
+class RoiSubsetState(RoiSubsetStateNd):
     """
     A subset defined as the set of points in two dimensions that lie inside
     a region of interest (ROI).
@@ -489,108 +595,42 @@ class RoiSubsetState(SubsetState):
         The data attribute on the y axis.
     roi : :class:`~glue.core.roi.Roi`
         The region of interest.
+    pretransform: callable, optional
+        A function that can be optionally applied to the data before checking points against the region.
     """
 
     @contract(xatt='isinstance(ComponentID)', yatt='isinstance(ComponentID)')
-    def __init__(self, xatt=None, yatt=None, roi=None):
-        super(RoiSubsetState, self).__init__()
-        self._xatt = xatt
-        self._yatt = yatt
-        self._roi = roi
+    def __init__(self, xatt=None, yatt=None, roi=None, pretransform=None):
+        super(RoiSubsetState, self).__init__(atts=[xatt, yatt], roi=roi, pretransform=pretransform)
 
     @property
     def xatt(self):
         """
         The data attribute on the x axis.
         """
-        return self._xatt
+        return self._atts[0]
 
     @xatt.setter
     def xatt(self, value):
-        self._xatt = value
+        self._atts[0] = value
 
     @property
     def yatt(self):
         """
         The data attribute on the y axis.
         """
-        return self._yatt
+        return self._atts[1]
 
     @yatt.setter
     def yatt(self, value):
-        self._yatt = value
-
-    @property
-    def roi(self):
-        """
-        The region of interest.
-        """
-        return self._roi
-
-    @roi.setter
-    def roi(self, value):
-        self._roi = value
-
-    @property
-    def attributes(self):
-        return (self.xatt, self.yatt)
-
-    @contract(data='isinstance(Data)', view='array_view')
-    def to_mask(self, data, view=None):
-
-        # TODO: make sure that pixel components don't actually take up much
-        #       memory and are just views
-
-        x = data[self.xatt, view]
-        y = data[self.yatt, view]
-
-        if (x.ndim == data.ndim and
-            self.xatt in data.pixel_component_ids and
-                self.yatt in data.pixel_component_ids):
-
-            # This is a special case - the ROI is defined in pixel space, so we
-            # can apply it to a single slice and then broadcast it to all other
-            # dimensions. We start off by extracting a slice which takes only
-            # the first elements of all dimensions except the attributes in
-            # question, for which we take all the elements. We need to preserve
-            # the dimensionality of the array, hence the use of slice(0, 1).
-            # Note that we can only do this if the view (if present) preserved
-            # the dimensionality, which is why we checked that x.ndim == data.ndim
-
-            subset = []
-            for i in range(data.ndim):
-                if i == self.xatt.axis or i == self.yatt.axis:
-                    subset.append(slice(None))
-                else:
-                    subset.append(slice(0, 1))
-
-            x_slice = x[tuple(subset)]
-            y_slice = y[tuple(subset)]
-
-            if self.roi.defined():
-                result = self.roi.contains(x_slice, y_slice)
-            else:
-                result = np.zeros(x_slice.shape, dtype=bool)
-
-            result = broadcast_to(result, x.shape)
-
-        else:
-
-            if self.roi.defined():
-                result = self.roi.contains(x, y)
-            else:
-                result = np.zeros(x.shape, dtype=bool)
-
-        if result.shape != x.shape:
-            raise ValueError("Unexpected error: boolean mask has incorrect dimensions")
-
-        return result
+        self._atts[1] = value
 
     def copy(self):
         result = RoiSubsetState()
         result.xatt = self.xatt
         result.yatt = self.yatt
         result.roi = self.roi
+        result.pretransform = self.pretransform
         return result
 
 
@@ -1510,12 +1550,12 @@ class InequalitySubsetState(SubsetState):
         if operator not in VALID_INEQUALTIY_OPS:
             raise TypeError("Invalid boolean operator: %s" % operator)
         if not isinstance(left, (ComponentID, numbers.Number,
-                                 ComponentLink, six.string_types)):
+                                 ComponentLink, str)):
             raise TypeError("Input must be ComponentID or NumberType or string: %s"
                             % type(left))
 
         if not isinstance(right, (ComponentID, numbers.Number,
-                                  ComponentLink, six.string_types)):
+                                  ComponentLink, str)):
             raise TypeError("Input must be ComponentID or NumberType or string: %s"
                             % type(right))
         self._left = left
@@ -1564,12 +1604,12 @@ class InequalitySubsetState(SubsetState):
         if view is None:
             view = Ellipsis
 
-        if isinstance(self._left, (numbers.Number, six.string_types)):
+        if isinstance(self._left, (numbers.Number, str)):
             left = self._left
         else:
             left = data[self._left, view]
 
-        if isinstance(self._right, (numbers.Number, six.string_types)):
+        if isinstance(self._right, (numbers.Number, str)):
             right = self._right
         else:
             right = data[self._right, view]
@@ -1708,7 +1748,7 @@ class FloodFillSubsetState(MaskSubsetState):
                    context.object(rec['threshold']))
 
 
-class RoiSubsetState3d(SubsetState):
+class RoiSubsetState3d(RoiSubsetStateNd):
     """
     A subset defined as the set of points in three dimensions that lie inside
     a 3-d region of interest (ROI).
@@ -1725,82 +1765,46 @@ class RoiSubsetState3d(SubsetState):
         The data attribute on the z axis.
     roi : :class:`~glue.core.roi.Roi`
         The region of interest (which should implement ``contains3d``)
+    pretransform: callable, optional
+        A function that can be optionally applied to the data before checking points against the region.
     """
 
     @contract(xatt='isinstance(ComponentID)', yatt='isinstance(ComponentID)', zatt='isinstance(ComponentID)')
-    def __init__(self, xatt=None, yatt=None, zatt=None, roi=None):
-        super(RoiSubsetState3d, self).__init__()
-        self._xatt = xatt
-        self._yatt = yatt
-        self._zatt = zatt
-        self._roi = roi
+    def __init__(self, xatt=None, yatt=None, zatt=None, roi=None, pretransform=None):
+        super(RoiSubsetState3d, self).__init__(atts=[xatt, yatt, zatt], roi=roi, pretransform=pretransform)
 
     @property
     def xatt(self):
         """
         The data attribute on the x axis.
         """
-        return self._xatt
+        return self._atts[0]
 
     @xatt.setter
     def xatt(self, value):
-        self._xatt = value
+        self._atts[0] = value
 
     @property
     def yatt(self):
         """
         The data attribute on the y axis.
         """
-        return self._yatt
+        return self._atts[1]
 
     @yatt.setter
     def yatt(self, value):
-        self._yatt = value
+        self._atts[1] = value
 
     @property
     def zatt(self):
         """
         The data attribute on the z axis.
         """
-        return self._zatt
+        return self._atts[2]
 
     @zatt.setter
     def zatt(self, value):
-        self._zatt = value
-
-    @property
-    def roi(self):
-        """
-        The region of interest.
-        """
-        return self._roi
-
-    @roi.setter
-    def roi(self, value):
-        self._roi = value
-
-    @property
-    def attributes(self):
-        return (self.xatt, self.yatt, self.zatt)
-
-    @contract(data='isinstance(Data)', view='array_view')
-    def to_mask(self, data, view=None):
-
-        # TODO: make sure that pixel components don't actually take up much
-        #       memory and are just views
-        x = data[self.xatt, view]
-        y = data[self.yatt, view]
-        z = data[self.zatt, view]
-
-        if self.roi.defined():
-            result = self.roi.contains3d(x, y, z)
-        else:
-            result = np.zeros(x.shape, dtype=bool)
-
-        if result.shape != x.shape:
-            raise ValueError("Unexpected error: boolean mask has incorrect dimensions")
-
-        return result
+        self._atts[2] = value
 
     def copy(self):
         result = RoiSubsetState3d()
@@ -1808,20 +1812,24 @@ class RoiSubsetState3d(SubsetState):
         result.yatt = self.yatt
         result.zatt = self.zatt
         result.roi = self.roi
+        result.pretransform = self.pretransform
         return result
 
     def __gluestate__(self, context):
         return dict(xatt=context.id(self.xatt),
                     yatt=context.id(self.yatt),
                     zatt=context.id(self.zatt),
-                    roi=context.id(self.roi))
+                    roi=context.id(self.roi),
+                    pretransform=context.id(self.pretransform))
 
     @classmethod
     def __setgluestate__(cls, rec, context):
+        pretrans = rec['pretransform'] if 'pretransform' in rec else None
         return RoiSubsetState3d(context.object(rec['xatt']),
                                 context.object(rec['yatt']),
                                 context.object(rec['zatt']),
-                                context.object(rec['roi']))
+                                context.object(rec['roi']),
+                                context.object(pretrans))
 
 
 @contract(subsets='list(isinstance(Subset))', returns=Subset)
@@ -1842,13 +1850,13 @@ def combine_multiple(subsets, operator):
         return combined
 
 
-def roi_to_subset_state(roi, x_att=None, y_att=None, x_categories=None, y_categories=None):
+def roi_to_subset_state(roi, x_att=None, y_att=None, x_categories=None, y_categories=None, use_pretransform=False):
     """
     Given a 2D ROI and attributes on the x and y axis, determine the
     corresponding subset state.
     """
 
-    if isinstance(roi, RangeROI):
+    if isinstance(roi, RangeROI) and not use_pretransform:
 
         if roi.ori == 'x':
             att = x_att
@@ -1952,11 +1960,14 @@ def roi_to_subset_state(roi, x_att=None, y_att=None, x_categories=None, y_catego
 
     else:
 
-        # The selection is polygon-like and components are numerical
+        # The selection is polygon-like or requires a pretransform and components are numerical
+
+        if not isinstance(roi, (PolygonalROI, RectangularROI, CircularROI, EllipticalROI, RangeROI)):
+            roi = PolygonalROI(*roi.to_polygon())
 
         subset_state = RoiSubsetState()
         subset_state.xatt = x_att
         subset_state.yatt = y_att
-        subset_state.roi = PolygonalROI(*roi.to_polygon())
+        subset_state.roi = roi
 
         return subset_state
