@@ -1,22 +1,19 @@
 # pylint: disable=I0011,W0613,W0201,W0212,E1101,E1103,R0903,R0904
 
-from __future__ import absolute_import, division, print_function
-
 import pytest
 import numpy as np
 from numpy.testing import assert_equal, assert_allclose
-from mock import MagicMock
+from unittest.mock import MagicMock
 from astropy.utils import NumpyRNGContext
 
-from glue.external import six
 from glue import core
 from glue.utils import broadcast_to
 
 from ..component import Component, DerivedComponent, CategoricalComponent, DateTimeComponent
 from ..component_id import ComponentID
 from ..component_link import ComponentLink, CoordinateComponentLink, BinaryComponentLink
-from ..coordinates import Coordinates
-from ..data import Data, pixel_label
+from ..coordinates import Coordinates, IdentityCoordinates
+from ..data import Data, pixel_label, BaseCartesianData
 from ..link_helpers import LinkSame
 from ..data_collection import DataCollection
 from ..exceptions import IncompatibleAttribute
@@ -33,10 +30,13 @@ from .test_state import clone
 
 class _TestCoordinates(Coordinates):
 
-    def pixel2world(self, *args):
+    def __init__(self):
+        super().__init__(pixel_n_dim=2, world_n_dim=2)
+
+    def pixel_to_world_values(self, *args):
         return [(i + 2.) * a for i, a in enumerate(args)]
 
-    def world2pixel(self, *args):
+    def world_to_pixel_values(self, *args):
         return [a / (i + 2.) for i, a in enumerate(args)]
 
 
@@ -96,12 +96,8 @@ class TestData(object):
         comp.data.shape = (3, 2)
         with pytest.raises(TypeError) as exc:
             self.data.add_component(comp("junk label"))
-        if six.PY3:
-            assert exc.value.args[0] == ("add_component() missing 1 required "
-                                         "positional argument: 'label'")
-        else:
-            assert exc.value.args[0] == ("add_component() takes exactly 3 "
-                                         "arguments (2 given)")
+        assert exc.value.args[0] == ("add_component() missing 1 required "
+                                        "positional argument: 'label'")
 
     def test_get_getitem_incompatible_attribute(self):
         cid = ComponentID('bad')
@@ -371,8 +367,8 @@ class TestData(object):
 
         data['z'] = data.id['x'] + 1
 
-        # There should be five components: x, y, z, pixel, and world
-        assert len(data.components) == 5
+        # There should be four components: x, y, z, and pixel
+        assert len(data.components) == 4
 
     def test_remove_derived_dependency(self):
 
@@ -395,14 +391,14 @@ class TestData(object):
         e_id = data.id['e']
         f_id = data.id['f']
 
-        # There should be five components: pixel, world, a, b, c, d, e, f
-        assert len(data.components) == 8
+        # There should be seven components: pixel, a, b, c, d, e, f
+        assert len(data.components) == 7
 
         data.remove_component(data.id['d'])
 
         # This should also remove e and f since they depend on d
 
-        assert len(data.components) == 5
+        assert len(data.components) == 4
 
         assert a_id in data.components
         assert b_id in data.components
@@ -413,7 +409,8 @@ class TestData(object):
 
     def test_links_property(self):
 
-        data = Data(a=[1, 2, 3], b=[2, 3, 4], label='data1')
+        data = Data(a=[1, 2, 3], b=[2, 3, 4], label='data1',
+                    coords=IdentityCoordinates(n_dim=1))
 
         assert len(data.links) == 2
         assert isinstance(data.links[0], CoordinateComponentLink)
@@ -609,8 +606,8 @@ def test_foreign_pixel_components_not_in_visible():
 
     # currently, this is trivially satisfied since all coordinates are hidden
 
-    d1 = Data(x=[1], y=[2])
-    d2 = Data(w=[3], v=[4])
+    d1 = Data(x=[1], y=[2], coords=IdentityCoordinates(n_dim=1))
+    d2 = Data(w=[3], v=[4], coords=IdentityCoordinates(n_dim=1))
     dc = DataCollection([d1, d2])
     dc.add_link(LinkSame(d1.id['x'], d2.id['w']))
 
@@ -639,7 +636,6 @@ Main components:
  - y
 Coordinate components:
  - Pixel Axis 0 [x]
- - World 0
 """.strip()
 
 
@@ -660,7 +656,6 @@ Derived components:
  - z
 Coordinate components:
  - Pixel Axis 0 [x]
- - World 0
 """.strip()
 
 
@@ -795,13 +790,13 @@ def test_update_coords():
     # Make sure that when overriding coords, the world coordinate components
     # are updated.
 
-    data1 = Data(x=[1, 2, 3])
+    data1 = Data(x=[1, 2, 3], coords=IdentityCoordinates(n_dim=1))
 
     assert len(data1.components) == 3
 
     assert_equal(data1[data1.world_component_ids[0]], [0, 1, 2])
 
-    data2 = Data(x=[1, 2, 3])
+    data2 = Data(x=[1, 2, 3], coords=IdentityCoordinates(n_dim=1))
 
     assert len(data1.links) == 2
     assert len(data2.links) == 2
@@ -816,13 +811,17 @@ def test_update_coords():
 
     class CustomCoordinates(Coordinates):
 
-        def axis_label(self, axis):
-            return 'Custom {0}'.format(axis)
+        def __init__(self):
+            super().__init__(pixel_n_dim=1, world_n_dim=1)
 
-        def world2pixel(self, *world):
+        @property
+        def world_axis_names(self):
+            return ['Custom {0}'.format(axis) for axis in range(3)]
+
+        def world_to_pixel_values(self, *world):
             return tuple([0.4 * w for w in world])
 
-        def pixel2world(self, *pixel):
+        def pixel_to_world_values(self, *pixel):
             return tuple([2.5 * p for p in pixel])
 
     data1.coords = CustomCoordinates()
@@ -872,7 +871,7 @@ def test_compute_statistic_random_subset():
         result = data.compute_statistic('mean', data.id['x'], random_subset=5,
                                         subset_state=MaskSubsetState([0, 1, 0, 1, 1, 1, 0, 1, 0, 1],
                                                                      data.pixel_component_ids))
-        assert_allclose(result, 4.75)
+        assert_allclose(result, 5)
 
 
 def test_compute_statistic_empty_subset():
@@ -895,6 +894,103 @@ def test_compute_statistic_empty_subset():
     assert_equal(result, np.nan)
 
 
+def test_compute_statistic_efficient():
+
+    # Unit test to test the logic for dealing with accessing only a minimal
+    # region from the data based on the smallest array that covers a given
+    # subset state.
+
+    array = np.ones(10 * 20 * 30 * 40).reshape((10, 20, 40, 30))
+    array[3:5, 6:14, :, 10:21:2] += 1
+
+    class CustomData(Data):
+        def get_data(self, cid, view=None):
+            if cid.label == 'x':
+                self.elements_accessed = np.ones(self.shape)[view].sum()
+            else:
+                self.elements_accessed = 0
+            return super().get_data(cid, view=view)
+
+    data = CustomData(x=array, y=array)
+
+    subset_state = data.id['y'] > 1.5
+
+    # First test without view
+    result = data.compute_statistic('sum', data.id['x'], subset_state=subset_state)
+    assert_allclose(result, 7680)
+    assert data.elements_accessed == 7040
+
+    # Now apply a view which includes just one slice that covers the original area
+    result = data.compute_statistic('sum', data.id['x'], subset_state=subset_state,
+                                    view=[slice(0, 5)])
+    assert_allclose(result, 7680)
+    assert data.elements_accessed == 7040
+
+    # Make it so that the slice doesn't fully overlap with the subset
+    result = data.compute_statistic('sum', data.id['x'], subset_state=subset_state,
+                                    view=[slice(0, 4)])
+    assert_allclose(result, 3840)
+    assert data.elements_accessed == 3520
+
+    # And now make it so there is no overlap
+    # TODO: should this result be 0 instead of nan?
+    result = data.compute_statistic('sum', data.id['x'], subset_state=subset_state,
+                                    view=[slice(0, 3)])
+    assert_allclose(result, np.nan)
+    assert data.elements_accessed == 0
+
+    # Check what happens if we use an integer index that overlaps...
+    result = data.compute_statistic('sum', data.id['x'], subset_state=subset_state,
+                                    view=[3])
+    assert_allclose(result, 3840)
+    assert data.elements_accessed == 3520
+
+    # ... and one that doesn't
+    # TODO: should this result be 0 instead of nan?
+    result = data.compute_statistic('sum', data.id['x'], subset_state=subset_state,
+                                    view=[2])
+    assert_allclose(result, np.nan)
+    assert data.elements_accessed == 0
+
+    # Now try using a slice that has a step>1 - this should actually then
+    # bypass the efficient algorithm
+    result = data.compute_statistic('sum', data.id['x'], subset_state=subset_state,
+                                    view=[slice(0, 5, 2)])
+    assert_allclose(result, 3840)
+    assert data.elements_accessed == 72000
+
+    # Finally we can do a complex mix of options
+    result = data.compute_statistic('sum', data.id['x'], subset_state=subset_state,
+                                    view=(slice(0, 5), slice(3, 10), 20, slice(None)))
+    assert_allclose(result, 96)
+    assert data.elements_accessed == 88
+
+
+def test_compute_statistic_shape():
+
+    # The compute_statistic method has some code that helps it be more efficient
+    # with subsets, but we need to make sure that the final result has the same
+    # shape as if we didn't have those optimizations.
+
+    array = np.ones(10 * 20 * 30).reshape((10, 20, 30))
+    array[3:5, 6:14, 10:21] += 1
+
+    data = Data(x=array, y=array)
+
+    subset_state = data.id['y'] > 1.5
+
+    result = data.compute_statistic('sum', data.id['x'], subset_state=subset_state)
+    assert np.isscalar(result)
+
+    result = data.compute_statistic('sum', data.id['x'], subset_state=subset_state,
+                                    axis=1)
+    assert result.shape == (10, 30)
+
+    result = data.compute_statistic('sum', data.id['x'], subset_state=subset_state,
+                                    axis=(0, 2))
+    assert result.shape == (20,)
+
+
 def test_compute_histogram_log():
 
     # Make sure that the returned histogram is NaN everywhere if either of the
@@ -919,3 +1015,54 @@ def test_compute_histogram_log():
     data = Data(x=np.ones(10), y=np.ones(10))
     result = data.compute_histogram([data.id['x'], data.id['y']], range=[[1, 3], [-3, 5]], bins=[2, 3], log=[True, True])
     assert result.shape == (2, 3) and np.sum(result) == 0
+
+
+def test_compute_histogram_dask():
+
+    # Make sure that compute_histogram works for dask arrays
+
+    da = pytest.importorskip('dask.array')
+
+    data = Data(x=da.arange(10))
+
+    result = data.compute_histogram([data.id['x']], range=[[-0.5, 11.75]], bins=[2])
+    assert_allclose(result, [6, 4])
+
+    result = data.compute_histogram([data.id['x']], range=[[-0.5, 11.25]], bins=[2], subset_state=data.id['x'] > 4.5)
+    assert_allclose(result, [1, 4])
+
+
+def test_base_cartesian_data_coords():
+
+    # Make sure that world_component_ids works in both the case where
+    # coords is not defined and when it is defined.
+
+    class CustomData(BaseCartesianData):
+
+        def get_kind(self):
+            pass
+
+        def compute_histogram(self):
+            pass
+
+        def compute_statistic(self):
+            pass
+
+        def get_mask(self):
+            pass
+
+        @property
+        def shape(self):
+            return (1, 4, 3)
+
+        @property
+        def main_components(self):
+            return []
+
+    data1 = CustomData()
+    assert len(data1.pixel_component_ids) == 3
+    assert len(data1.world_component_ids) == 0
+
+    data2 = CustomData(coords=IdentityCoordinates(n_dim=3))
+    assert len(data2.pixel_component_ids) == 3
+    assert len(data2.world_component_ids) == 3
