@@ -1,13 +1,11 @@
 # pylint: disable=I0011,W0613,W0201,W0212,E1101,E1103
 
-from __future__ import absolute_import, division, print_function
-
 import pytest
 import numpy as np
-from mock import MagicMock
+from unittest.mock import MagicMock
 from numpy.testing import assert_array_equal, assert_equal
 
-from ..coordinates import Coordinates
+from ..coordinates import IdentityCoordinates
 from ..component_link import ComponentLink
 from ..link_helpers import LinkSame
 from ..data import Data, Component, ComponentID, DerivedComponent
@@ -91,6 +89,11 @@ class TestDataCollection(object):
         self.dc.remove(self.data)
         assert self.data not in self.dc
 
+    def test_clear(self):
+        self.dc.append(self.data)
+        self.dc.clear()
+        assert self.data not in self.dc
+
     def test_ignore_multi_remove(self):
         self.dc.append(self.data)
         self.dc.remove(self.data)
@@ -122,6 +125,24 @@ class TestDataCollection(object):
         self.dc.append(self.data)
         assert self.dc[0] is self.data
 
+    def test_get_item_str(self):
+
+        data = Data(label='test')
+        self.dc.append(data)
+
+        assert self.dc['test'] is data
+
+        with pytest.raises(ValueError) as exc:
+            self.dc['spam']
+        assert exc.value.args[0] == "No data found with the label 'spam'"
+
+        data2 = Data(label='test')
+        self.dc.append(data2)
+
+        with pytest.raises(ValueError) as exc:
+            self.dc['test']
+        assert exc.value.args[0] == "Several datasets were found with the label 'test'"
+
     def test_iter(self):
         self.dc.append(self.data)
         assert set(self.dc) == set([self.data])
@@ -279,7 +300,7 @@ class TestDataCollection(object):
         dc.merge(x, y)
         merged = dc[0]
 
-        assert len(merged.coordinate_components) == 2
+        assert len(merged.coordinate_components) == 1
         assert y.pixel_component_ids[0] not in merged.components
 
     def test_merge_forbids_single_argument(self):
@@ -359,8 +380,8 @@ class TestDataCollection(object):
         y = Data(y=[2, 3, 4])
         dc = DataCollection([x, y])
 
-        x.coords = Coordinates()
-        y.coords = Coordinates()
+        x.coords = IdentityCoordinates(n_dim=1)
+        y.coords = IdentityCoordinates(n_dim=1)
 
         dc.merge(x, y)
 
@@ -375,12 +396,13 @@ class TestDataCollection(object):
         y = Data(y=[2, 3, 4])
         dc = DataCollection([x, y])
 
-        class CustomCoordinates(Coordinates):
-            def axis_label(self, axis):
-                return 'Custom {0}'.format(axis)
+        class CustomCoordinates(IdentityCoordinates):
+            @property
+            def world_axis_names(self):
+                return ['Custom {0}'.format(axis) for axis in range(self.world_n_dim)]
 
-        x.coords = CustomCoordinates()
-        y.coords = CustomCoordinates()
+        x.coords = CustomCoordinates(n_dim=1)
+        y.coords = CustomCoordinates(n_dim=1)
 
         dc.merge(x, y)
 
@@ -424,8 +446,8 @@ class TestDataCollection(object):
         # This tests that the separation of internal vs external links is
         # preserved in session files.
 
-        d1 = Data(a=[1, 2, 3])
-        d2 = Data(b=[2, 3, 4])
+        d1 = Data(a=[1, 2, 3], coords=IdentityCoordinates(n_dim=1))
+        d2 = Data(b=[2, 3, 4], coords=IdentityCoordinates(n_dim=1))
 
         dc = DataCollection([d1, d2])
         dc.add_link(ComponentLink([d2.id['b']], d1.id['a']))
