@@ -1,9 +1,7 @@
 # pylint: disable=I0011,W0613,W0201,W0212,E1101,E1103
 
-from __future__ import absolute_import, division, print_function
-
 import pytest
-from mock import MagicMock
+from unittest.mock import MagicMock
 
 from ..data import Data
 from ..data_collection import DataCollection
