@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 import json
 from io import BytesIO
 
@@ -7,10 +5,10 @@ import pytest
 import numpy as np
 from numpy.testing import assert_equal
 
-from glue.external import six
 from glue import core
 from glue.core.component import CategoricalComponent, DateTimeComponent
 from glue.tests.helpers import requires_astropy, make_file
+from glue.config import session_patch
 
 from ..data_factories import load_data
 from ..data_factories.tests.test_fits import TEST_FITS_DATA
@@ -35,7 +33,7 @@ def containers_equal(c1, c2):
     """Check that two container-like items have the same contents,
     ignoring differences relating to the type of container
     """
-    if isinstance(c1, six.string_types):
+    if isinstance(c1, str):
         return c1 == c2
 
     try:
@@ -104,6 +102,19 @@ def test_data_style():
     assert d2.style.color == 'blue'
 
 
+def test_user_patch_is_called():
+    @session_patch()
+    def a_patch(session):
+        session["__main__"]["label"] = 'has_changed'
+
+    d = core.Data(x=[1, 2, 3], label='testing')
+    d2 = clone(d)
+
+    session_patch._members.pop(-1)  # clean registry
+
+    assert d2.label == 'has_changed'
+
+
 @requires_astropy
 def test_data_factory():
     with make_file(TEST_FITS_DATA, '.fits', decompress=True) as infile:
