@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 import logging
 from itertools import count
 from functools import partial
@@ -8,9 +6,9 @@ from functools import partial
 import numpy as np
 
 from matplotlib.ticker import AutoLocator, MaxNLocator, LogLocator
-from matplotlib.ticker import (LogFormatterMathtext, ScalarFormatter,
-                               FuncFormatter)
+from matplotlib.ticker import LogFormatterMathtext, ScalarFormatter, FuncFormatter
 from matplotlib.dates import AutoDateLocator, AutoDateFormatter
+from matplotlib.projections.polar import ThetaFormatter, ThetaLocator
 
 from glue.utils import nanmin, nanmax
 
@@ -336,7 +334,7 @@ def tick_linker(all_categories, pos, *args):
             return ''
 
 
-def update_ticks(axes, coord, kinds, is_log, categories):
+def update_ticks(axes, coord, kinds, is_log, categories, projection='rectilinear'):
     """
     Changes the axes to have the proper tick formatting based on the type of
     component.
@@ -353,8 +351,15 @@ def update_ticks(axes, coord, kinds, is_log, categories):
         A list of components that are plotted along this axis
     if_log : boolean
         Whether the axis has a log-scale
+    projection: str
+        The name of the matplotlib projection for the axes object. Defaults to 'rectilinear'.
+        Currently only the scatter viewer supports different projections.
     """
 
+    # Short circuit the full-sphere projections
+    if projection in ['aitoff', 'hammer', 'mollweide', 'lambert']:
+        return
+
     if coord == 'x':
         axis = axes.xaxis
     elif coord == 'y':
@@ -380,6 +385,10 @@ def update_ticks(axes, coord, kinds, is_log, categories):
         formatter = FuncFormatter(format_func)
         axis.set_major_locator(locator)
         axis.set_major_formatter(formatter)
+    # Have to treat the theta axis of polar plots differently
+    elif projection == 'polar' and coord == 'x':
+        axis.set_major_locator(ThetaLocator(AutoLocator()))
+        axis.set_major_formatter(ThetaFormatter())
     else:
         axis.set_major_locator(AutoLocator())
         axis.set_major_formatter(ScalarFormatter())
