@@ -1,29 +1,34 @@
-from __future__ import absolute_import, division, print_function
-
 from qtpy.QtCore import Qt
 from qtpy import QtWidgets
-from glue.utils.qt import QMessageBoxPatched as QMessageBox, set_cursor_cm
+from glue.utils.qt import set_cursor_cm
 
 __all__ = ['data_wizard', 'GlueDataDialog']
 
 
-def data_wizard():
-    """ QT Dialog to load a file into a new data object
+def data_wizard(mode='files'):
+    """
+    Qt Dialog to load a file into a new data object
 
-    Returns:
-       A list of new data objects. Returns an empty list if
-       selection is canceled.
+    Parameters
+    ----------
+    mode : {'files', 'directories'}
+        Whether to allow the user to select files or directories.
+
+    Returns
+    -------
+    A list of new data objects. Returns an empty list if
+    selection is canceled.
     """
     def report_error(error, factory, curfile):
         import traceback
-        retry = QMessageBox.Retry
-        cancel = QMessageBox.Cancel
+        retry = QtWidgets.QMessageBox.Retry
+        cancel = QtWidgets.QMessageBox.Cancel
         buttons = retry | cancel
         detail = traceback.format_exc()
         msg = "\n".join(["Could not load %s (wrong load method?)" % curfile,
                          "File load method: %s" % factory.label])
         detail = "\n\n".join(["Error message: %s" % error, detail])
-        mb = QMessageBox(QMessageBox.Critical, "Data Load Error", msg)
+        mb = QtWidgets.QMessageBox(QtWidgets.QMessageBox.Critical, "Data Load Error", msg)
         mb.setDetailedText(detail)
         mb.setDefaultButton(cancel)
         mb.setStandardButtons(buttons)
@@ -31,7 +36,7 @@ def data_wizard():
         return ok == retry
 
     while True:
-        gdd = GlueDataDialog()
+        gdd = GlueDataDialog(mode=mode)
         try:
             result = gdd.load_data()
             break
@@ -44,13 +49,18 @@ def data_wizard():
 
 class GlueDataDialog(object):
 
-    def __init__(self, parent=None):
+    def __init__(self, mode='files', parent=None):
         self._fd = QtWidgets.QFileDialog(parent)
         from glue.config import data_factory
         self.filters = [(f, self._filter(f))
                         for f in data_factory.members if not f.deprecated]
         self.setNameFilter()
-        self._fd.setFileMode(QtWidgets.QFileDialog.ExistingFiles)
+        if mode == 'files':
+            self._fd.setFileMode(QtWidgets.QFileDialog.ExistingFiles)
+        else:
+            self._fd.setFileMode(QtWidgets.QFileDialog.Directory)
+            self._fd.setOption(QtWidgets.QFileDialog.ShowDirsOnly, True)
+
         self._curfile = ''
         try:
             self._fd.setOption(
@@ -101,12 +111,25 @@ class GlueDataDialog(object):
         paths, fac = self._get_paths_and_factory()
         result = []
 
+        # Check that the user didn't select a .glu file by mistake
+        for path in paths:
+            if path.endswith('.glu'):
+                mb = QtWidgets.QMessageBox(QtWidgets.QMessageBox.Critical,
+                                           "Error loading data",
+                                           "It looks like you have selected "
+                                           "a .glu session file. You should open "
+                                           "this using 'Open Session' under the "
+                                           "'File' menu instead")
+                mb.exec_()
+                return []
+
         with set_cursor_cm(Qt.WaitCursor):
             for path in paths:
                 self._curfile = path
                 d = load_data(path, factory=fac.function)
                 if not isinstance(d, list):
-                    d.label = data_label(path)
+                    if not d.label:
+                        d.label = data_label(path)
                     d = [d]
                 result.extend(d)
 
