@@ -1,6 +1,4 @@
-from __future__ import absolute_import, division, print_function
-
-from mock import patch
+from unittest.mock import patch
 from matplotlib import cm
 
 from glue.core import Data, DataCollection
