@@ -1,9 +1,7 @@
-from __future__ import absolute_import, division, print_function
-
 import pytest
 import numpy as np
 import pandas as pd
-from mock import patch
+from unittest.mock import patch
 
 from .. import qglue
 from ..core import BaseCartesianData, Data
