@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 import warnings
 
 import numpy as np
@@ -8,15 +6,12 @@ from numpy.lib.stride_tricks import as_strided
 import pandas as pd
 import bottleneck as bt
 
-from glue.external.six import string_types
-from glue.external.six.moves import range
-
-
 __all__ = ['unique', 'shape_to_string', 'view_shape', 'stack_view',
            'coerce_numeric', 'check_sorted', 'broadcast_to', 'unbroadcast',
            'iterate_chunks', 'combine_slices', 'nanmean', 'nanmedian', 'nansum',
            'nanmin', 'nanmax', 'format_minimal', 'compute_statistic',
-           'categorical_ndarray', 'index_lookup', 'ensure_numerical']
+           'categorical_ndarray', 'index_lookup', 'ensure_numerical',
+           'broadcast_arrays_minimal', 'random_views_for_dask_array']
 
 
 def unbroadcast(array):
@@ -28,13 +23,20 @@ def unbroadcast(array):
     for more details.
     """
 
-    if array.ndim == 0:
+    if array.ndim == 0 or not hasattr(array, 'strides'):
         return array
 
     new_shape = np.where(np.array(array.strides) == 0, 1, array.shape)
     return as_strided(array, shape=new_shape)
 
 
+def broadcast_arrays_minimal(*arrays):
+    """
+    Unbroadcast arrays then broadcast to smallest common shape.
+    """
+    return np.broadcast_arrays(*[unbroadcast(array) for array in arrays])
+
+
 def unique(array):
     """
     Return the unique elements of the array U, as well as
@@ -92,7 +94,7 @@ def stack_view(shape, *views):
     shp = tuple(slice(0, s, 1) for s in shape)
     result = np.broadcast_arrays(*np.ogrid[shp])
     for v in views:
-        if isinstance(v, string_types) and v == 'transpose':
+        if isinstance(v, str) and v == 'transpose':
             result = [r.T for r in result]
             continue
 
@@ -596,3 +598,36 @@ def index_lookup(data, items):
     result = np.zeros(ndata, dtype=float) * np.nan
     result[np.array(m.row)] = m.cat_row
     return result
+
+
+def random_views_for_dask_array(array, n_random_samples, n_chunks):
+    """
+    Return a list of views to extract random values from a dask array in an
+    efficient way taking into account the chunk layout. This will return
+    n_chunks views such that all views together add up to approximately
+    n_random_samples samples.
+    """
+
+    # Find the indices of the chunks to extract
+    indices = [np.random.randint(dimsize, size=n_chunks) for dimsize in array.numblocks]
+
+    # Determine the boundaries of chunks along each dimension
+    chunk_indices = [np.hstack([0, np.cumsum([size for size in sizes])]) for sizes in array.chunks]
+
+    n_per_chunk = n_random_samples // n_chunks
+
+    all_slices = []
+    for ichunk in range(n_chunks):
+        slices = []
+        remaining_size = n_per_chunk
+        for idim in range(array.ndim):
+            start = chunk_indices[idim][indices[idim][ichunk]]
+            stop = chunk_indices[idim][indices[idim][ichunk] + 1]
+            if stop - start > remaining_size:
+                stop = start + remaining_size
+            slices.append(slice(start, stop))
+            remaining_size //= (stop - start)
+            remaining_size = max(1, remaining_size)
+        all_slices.append(tuple(slices))
+
+    return all_slices
