@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 import logging
 import warnings
 from functools import wraps
@@ -7,6 +5,8 @@ from functools import wraps
 import numpy as np
 import matplotlib.units as units
 import matplotlib.dates as dates
+from matplotlib.legend_handler import HandlerBase
+from matplotlib.patches import Rectangle
 
 # We avoid importing matplotlib up here otherwise Matplotlib and therefore Qt
 # get imported as soon as glue.utils is imported.
@@ -23,7 +23,7 @@ __all__ = ['renderless_figure', 'all_artists', 'new_artists', 'remove_artists',
 
 def renderless_figure():
     # Matplotlib figure that skips the render step, for test speed
-    from mock import MagicMock
+    from unittest.mock import MagicMock
     import matplotlib.pyplot as plt
     fig = plt.figure()
     fig.canvas.draw = MagicMock()
@@ -333,6 +333,41 @@ def cache_axes(axes, toolbar):
     return cache
 
 
+class ColormapPatchHandler(HandlerBase):
+    def __init__(self, cmap, nb_subpatch=10, xpad=0.0, ypad=0.0):
+        """
+        A custom legend handler to represent 2D dataset coded in colormaps
+
+        Parameters
+        ----------
+        cmap : `~matplotlib.colors.colormap`
+            The matplotlib colormap to use
+        nb_subpatch : int, optional
+            The number of stripes to use to represent the colormap.
+            The default is 10.
+        xpad : float, optional
+            Padding in the x direction. The default is 0.0.
+        ypad : float, optional
+            Padding in the y direction. The default is 0.0.
+
+        """
+        super().__init__(xpad, ypad)
+        self.nb_subpatch = nb_subpatch
+        self.cmap = cmap
+
+    def create_artists(self, legend, orig_handle,
+                       xdescent, ydescent, width, height, fontsize,
+                       trans):
+        collection = []
+        for i in range(self.nb_subpatch):
+            width_sub = width / self.nb_subpatch
+            x = xdescent + i * width_sub
+            collection.append(
+                Rectangle((x, ydescent), width_sub, height, transform=trans,
+                          facecolor=self.cmap(i / (self.nb_subpatch - 1)),
+                          edgecolor="none"))
+        return collection
+
 # In Matplotlib < 2.2, there is no datetime64 support, so we register a converter
 # here to deal with it with older versions.
 
