@@ -1,8 +1,6 @@
-from __future__ import absolute_import, division, print_function
+from unittest.mock import MagicMock
 
-from mock import MagicMock
-
-from glue.external.echo import CallbackProperty
+from echo import CallbackProperty
 from qtpy import QtGui
 
 from ..colors import qt_to_mpl_color, QColorBox, connect_color, QColormapCombo
