@@ -1,10 +1,8 @@
-from __future__ import absolute_import, division, print_function
-
 from collections import OrderedDict
 
 import numpy as np
 from matplotlib.axes import Axes
-from mock import MagicMock
+from unittest.mock import MagicMock
 from numpy.testing import assert_array_equal
 
 from glue.core.tests.test_state import clone
