@@ -1,7 +1,5 @@
 # pylint: disable=I0011,W0613,W0201,W0212,E1101,E1103
 
-from __future__ import absolute_import, division, print_function
-
 import os
 from collections import Counter
 
@@ -12,8 +10,8 @@ from numpy.testing import assert_equal, assert_allclose
 
 from glue.core.message import SubsetUpdateMessage
 from glue.core import HubListener, Data
-from glue.core.roi import XRangeROI
-from glue.core.subset import RangeSubsetState, CategoricalROISubsetState
+from glue.core.roi import XRangeROI, RectangularROI
+from glue.core.subset import RangeSubsetState, CategoricalROISubsetState, RoiSubsetState
 from glue import core
 from glue.app.qt import GlueApplication
 from glue.core.component_id import ComponentID
@@ -62,7 +60,7 @@ class TestHistogramViewer(object):
         # Check defaults when we add data
         self.viewer.add_data(self.data)
 
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:Coordinate components:Pixel Axis 0 [x]:World 0'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:Coordinate components:Pixel Axis 0 [x]'
 
         assert viewer_state.x_att is self.data.id['x']
         assert viewer_state.x_min == -1.1
@@ -146,7 +144,7 @@ class TestHistogramViewer(object):
 
     def test_remove_data(self):
         self.viewer.add_data(self.data)
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:Coordinate components:Pixel Axis 0 [x]:World 0'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:Coordinate components:Pixel Axis 0 [x]'
         self.data_collection.remove(self.data)
         assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == ''
 
@@ -160,7 +158,7 @@ class TestHistogramViewer(object):
         self.viewer.add_data(self.data)
         self.data.add_component([3, 4, 1, 2], 'z')
         assert self.viewer.state.x_att is self.data.id['x']
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]:World 0'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]'
 
     def test_nonnumeric_first_component(self):
         # regression test for #208. Shouldn't complain if
@@ -447,7 +445,7 @@ class TestHistogramViewer(object):
         test = ComponentID('test')
         self.data.update_id(self.viewer.state.x_att, test)
         assert self.viewer.state.x_att is test
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:test:y:Coordinate components:Pixel Axis 0 [x]:World 0'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:test:y:Coordinate components:Pixel Axis 0 [x]'
 
     def test_nbin_override_persists_over_numerical_attribute_change(self):
 
@@ -667,3 +665,70 @@ class TestHistogramViewer(object):
         assert [x.get_text() for x in viewer.axes.xaxis.get_ticklabels()] == ['', 'a', 'b', 'c', '']
 
         ga.close()
+
+    def test_legend(self):
+        from matplotlib.colors import to_hex
+
+        viewer_state = self.viewer.state
+
+        self.viewer.add_data(self.data)
+        self.viewer.state.legend.visible = True
+
+        handles, labels, handler_dict = self.viewer.get_handles_legend()
+        assert len(handles) == 1
+        assert labels[0] == 'd1'
+
+        self.data_collection.new_subset_group('test', self.data.id['x'] > 1)
+        assert len(viewer_state.layers) == 2
+        handles, labels, handler_dict = self.viewer.get_handles_legend()
+        assert len(handles) == 2
+        assert labels[1] == 'test'
+
+        assert to_hex(handles[1].get_facecolor()) == viewer_state.layers[1].color
+
+
+def test_with_dask_array():
+
+    # Regression test for a bug that caused the histogram to now work when
+    # making spatial selections on a cube represented as a dask array
+
+    da = pytest.importorskip('dask.array')
+
+    data = Data(x=da.arange(1000).reshape((10, 10, 10)), label='d1')
+
+    app = GlueApplication()
+    session = app.session
+    hub = session.hub
+
+    data_collection = session.data_collection
+    data_collection.append(data)
+
+    viewer = app.new_data_viewer(HistogramViewer)
+
+    viewer.add_data(data)
+
+    viewer.state.hist_n_bin = 1
+
+    process_events(0.5)
+
+    assert len(viewer.layers) == 1
+    assert viewer.layers[0].enabled
+
+    zid, yid, xid = data.pixel_component_ids
+
+    subset_state = RoiSubsetState(xatt=xid, yatt=yid,
+                                  roi=RectangularROI(xmin=3.5, xmax=5.5, ymin=3.7, ymax=7.5))
+
+    data_collection.new_subset_group(subset_state=subset_state, label='subset')
+
+    assert len(viewer.layers) == 2
+    assert viewer.layers[0].enabled
+    assert viewer.layers[1].enabled
+
+    assert viewer.state.layers[0].histogram[1][0] == 1000.
+    assert viewer.state.layers[1].histogram[1][0] == 80.
+
+    viewer.close()
+    viewer = None
+    app.close()
+    app = None
