@@ -1,10 +1,8 @@
-from __future__ import absolute_import, division, print_function
-
 import numpy as np
 
 from glue.core import BaseData, Subset
 
-from glue.external.echo import delay_callback
+from echo import delay_callback
 from glue.viewers.matplotlib.state import (MatplotlibDataViewerState,
                                            MatplotlibLayerState,
                                            DeferredDrawCallbackProperty as DDCProperty,
