@@ -1,10 +1,11 @@
-from __future__ import absolute_import, division, print_function
-
 import uuid
 import weakref
 
 import numpy as np
 
+import matplotlib.patches as mpatches
+import matplotlib.colors as mcolors
+
 from glue.utils import defer_draw, broadcast_to
 
 from glue.viewers.image.state import ImageLayerState, ImageSubsetLayerState
@@ -13,6 +14,7 @@ from glue.viewers.image.pixel_selection_subset_state import PixelSubsetState
 from glue.viewers.matplotlib.layer_artist import MatplotlibLayerArtist
 from glue.core.exceptions import IncompatibleAttribute
 from glue.utils import color2rgb
+from glue.utils.matplotlib import ColormapPatchHandler
 from glue.core import BaseData, HubListener
 from glue.core.message import (ComponentsChangedMessage,
                                ExternallyDerivableComponentsChangedMessage,
@@ -85,6 +87,22 @@ class ImageLayerArtist(BaseImageLayerArtist):
         else:
             return self.state.cmap
 
+    def get_handle_legend(self):
+        if self.enabled and self.state.visible:
+            # a fancier patch drawing the colormap
+            color = self.get_layer_color()
+            label = self.layer.label
+            if isinstance(color, mcolors.Colormap):
+                handle = mpatches.Patch(color=color(0.5))
+                # actual color of the handle is not important
+                handler = ColormapPatchHandler(color)
+            else:
+                handle = mpatches.Patch(color=color, alpha=self.layer.style.alpha)
+                handler = None  # default handler is good enough
+            return handle, label, handler
+        else:
+            return None, None, None
+
     def enable(self):
         if hasattr(self, 'composite_image'):
             self.composite_image.invalidate_cache()
@@ -97,10 +115,11 @@ class ImageLayerArtist(BaseImageLayerArtist):
             self.composite_image.invalidate_cache()
         ARRAY_CACHE.pop(self.state.uuid, None)
         PIXEL_CACHE.pop(self.state.uuid, None)
+        self.uuid = None
 
     def get_image_shape(self):
 
-        if self._viewer_state.reference_data is None or self._viewer_state.x_att is None or self._viewer_state.y_att is None:
+        if self.uuid is None or self._viewer_state.reference_data is None or self._viewer_state.x_att is None or self._viewer_state.y_att is None:
             return None
 
         x_axis = self._viewer_state.x_att.axis
@@ -112,6 +131,9 @@ class ImageLayerArtist(BaseImageLayerArtist):
 
     def get_image_data(self, bounds=None):
 
+        if self.uuid is None:
+            return None
+
         try:
             image = self.state.get_sliced_data(bounds=bounds)
         except (IncompatibleAttribute, IndexError):
@@ -155,7 +177,7 @@ class ImageLayerArtist(BaseImageLayerArtist):
     @defer_draw
     def _update_image(self, force=False, **kwargs):
 
-        if self.state.attribute is None or self.state.layer is None:
+        if self.uuid is None or self.state.attribute is None or self.state.layer is None:
             return
 
         changed = set() if force else self.pop_changed_properties()
@@ -260,7 +282,7 @@ class ImageSubsetLayerArtist(BaseImageLayerArtist):
 
         self.image_artist = imshow(self.axes, self.subset_array,
                                    origin='lower', interpolation='nearest',
-                                   vmin=0, vmax=1, aspect=self._viewer_state.aspect)
+                                   vmin=0, vmax=1, aspect='auto')
 
         self._line_x = self.axes.axvline(0)
         self._line_x.set_visible(False)
