@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 import numpy as np
 
 from qtpy import QtCore, QtWidgets
@@ -66,17 +64,6 @@ class StandaloneImageViewer(QtWidgets.QMainWindow):
 
         kwargs.setdefault('origin', 'upper')
 
-        if wcs is not None:
-            # In the following we force the color and linewith of the WCSAxes
-            # frame to be restored after calling reset_wcs. This can be removed
-            # once we support Astropy 1.3.1 or later.
-            color = self._axes.coords.frame.get_color()
-            linewidth = self._axes.coords.frame.get_linewidth()
-            self._axes.reset_wcs(wcs)
-            self._axes.coords.frame.set_color(color)
-            self._axes.coords.frame.set_linewidth(linewidth)
-            del color, linewidth
-
         self._composite.set('image', array=image, color=colormaps.members[0][1])
         self._im = imshow(self._axes, self._composite, **kwargs)
         self._im_array = image
