@@ -1,7 +1,5 @@
 # pylint: disable=I0011,W0613,W0201,W0212,E1101,E1103
 
-from __future__ import absolute_import, division, print_function
-
 import os
 import gc
 from collections import Counter
@@ -14,7 +12,7 @@ import numpy as np
 from numpy.testing import assert_allclose
 
 from glue.viewers.image.frb_artist import FRBArtist
-from glue.core.coordinates import Coordinates, WCSCoordinates
+from glue.core.coordinates import IdentityCoordinates
 from glue.core.message import SubsetUpdateMessage
 from glue.core import HubListener, Data
 from glue.core.roi import XRangeROI, RectangularROI
@@ -28,6 +26,7 @@ from glue.viewers.image.state import ImageLayerState, ImageSubsetLayerState, Agg
 from glue.core.link_helpers import LinkSame
 from glue.app.qt import GlueApplication
 from glue.core.fixed_resolution_buffer import ARRAY_CACHE, PIXEL_CACHE
+from glue.core.data_derived import IndexedData
 
 from ..data_viewer import ImageViewer
 
@@ -45,6 +44,19 @@ class TestImageCommon(BaseTestMatplotlibDataViewer):
     def test_double_add_ignored(self):
         pass
 
+    def test_update_data_processed_if_data_present(self):
+
+        # Patch for the main test of the same name - we need to explicilty set
+        # global_sync to True here for things to work correctly.
+
+        self.init_draw_count()
+        self.data_collection.append(self.data)
+        self.viewer.add_data(self.data)
+        ct0 = self.draw_count
+        self.viewer.state.layers[0].global_sync = True
+        self.data.style.color = 'blue'
+        assert self.draw_count > ct0
+
     def test_slice_change_single_draw(self):
 
         # Regression test for a bug that caused Matplotlib to draw once per
@@ -63,9 +75,14 @@ class TestImageCommon(BaseTestMatplotlibDataViewer):
         assert self.draw_count == 1
 
 
-class MyCoords(Coordinates):
-    def axis_label(self, i):
-        return ['Banana', 'Apple'][i]
+class MyCoords(IdentityCoordinates):
+
+    def __init__(self, n_dim=2):
+        super().__init__(n_dim=n_dim)
+
+    @property
+    def world_axis_names(self):
+        return ['Apple', 'Banana']
 
 
 class TestImageViewer(object):
@@ -81,9 +98,9 @@ class TestImageViewer(object):
 
         # Create data versions with WCS coordinates
         self.image1_wcs = Data(label='image1_wcs', x=self.image1['x'],
-                               coords=WCSCoordinates(wcs=WCS(naxis=2)))
+                               coords=WCS(naxis=2))
         self.hypercube_wcs = Data(label='hypercube_wcs', x=self.hypercube['x'],
-                                  coords=WCSCoordinates(wcs=WCS(naxis=4)))
+                                  coords=WCS(naxis=4))
 
         self.application = GlueApplication()
 
@@ -126,17 +143,17 @@ class TestImageViewer(object):
 
         self.viewer.add_data(self.image1)
 
-        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'Coordinate components:World 0:World 1'
-        assert combo_as_string(self.options_widget.ui.combosel_y_att_world) == 'Coordinate components:World 0:World 1'
+        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'Coordinate components:Pixel Axis 0 [y]:Pixel Axis 1 [x]'
+        assert combo_as_string(self.options_widget.ui.combosel_y_att_world) == 'Coordinate components:Pixel Axis 0 [y]:Pixel Axis 1 [x]'
 
-        assert self.viewer.axes.get_xlabel() == 'World 1'
-        assert self.viewer.state.x_att_world is self.image1.id['World 1']
+        assert self.viewer.axes.get_xlabel() == 'Pixel Axis 1 [x]'
+        assert self.viewer.state.x_att_world is self.image1.id['Pixel Axis 1 [x]']
         assert self.viewer.state.x_att is self.image1.pixel_component_ids[1]
         assert_allclose(self.viewer.state.x_min, -0.8419913419913423)
         assert_allclose(self.viewer.state.x_max, +1.8419913419913423)
 
-        assert self.viewer.axes.get_ylabel() == 'World 0'
-        assert self.viewer.state.y_att_world is self.image1.id['World 0']
+        assert self.viewer.axes.get_ylabel() == 'Pixel Axis 0 [y]'
+        assert self.viewer.state.y_att_world is self.image1.id['Pixel Axis 0 [y]']
         assert self.viewer.state.y_att is self.image1.pixel_component_ids[0]
         assert self.viewer.state.y_min == -0.5
         assert self.viewer.state.y_max == +1.5
@@ -185,8 +202,8 @@ class TestImageViewer(object):
     def test_combo_updates_with_component_add(self):
         self.viewer.add_data(self.image1)
         self.image1.add_component([[9, 9], [8, 8]], 'z')
-        assert self.viewer.state.x_att_world is self.image1.id['World 1']
-        assert self.viewer.state.y_att_world is self.image1.id['World 0']
+        assert self.viewer.state.x_att_world is self.image1.id['Pixel Axis 1 [x]']
+        assert self.viewer.state.y_att_world is self.image1.id['Pixel Axis 0 [y]']
         # TODO: there should be an easier way to do this
         layer_style_editor = self.viewer._view.layout_style_widgets[self.viewer.layers[0]]
         assert combo_as_string(layer_style_editor.ui.combosel_attribute) == 'x:y:z'
@@ -276,17 +293,17 @@ class TestImageViewer(object):
 
         self.viewer.add_data(self.hypercube)
 
-        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'Coordinate components:World 0:World 1:World 2:World 3'
-        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'Coordinate components:World 0:World 1:World 2:World 3'
+        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'Coordinate components:Pixel Axis 0:Pixel Axis 1:Pixel Axis 2:Pixel Axis 3'
+        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'Coordinate components:Pixel Axis 0:Pixel Axis 1:Pixel Axis 2:Pixel Axis 3'
 
-        assert self.viewer.axes.get_xlabel() == 'World 3'
-        assert self.viewer.state.x_att_world is self.hypercube.id['World 3']
+        assert self.viewer.axes.get_xlabel() == 'Pixel Axis 3'
+        assert self.viewer.state.x_att_world is self.hypercube.id['Pixel Axis 3']
         assert self.viewer.state.x_att is self.hypercube.pixel_component_ids[3]
         assert_allclose(self.viewer.state.x_min, -0.6839826839826846)
         assert_allclose(self.viewer.state.x_max, +4.6839826839826846)
 
-        assert self.viewer.axes.get_ylabel() == 'World 2'
-        assert self.viewer.state.y_att_world is self.hypercube.id['World 2']
+        assert self.viewer.axes.get_ylabel() == 'Pixel Axis 2'
+        assert self.viewer.state.y_att_world is self.hypercube.id['Pixel Axis 2']
         assert self.viewer.state.y_att is self.hypercube.pixel_component_ids[2]
         assert self.viewer.state.y_min == -0.5
         assert self.viewer.state.y_max == +3.5
@@ -302,7 +319,7 @@ class TestImageViewer(object):
 
         wcs = WCS(naxis=4)
         hypercube2 = Data()
-        hypercube2.coords = WCSCoordinates(wcs=wcs)
+        hypercube2.coords = wcs
         hypercube2.add_component(np.random.random((2, 3, 4, 5)), 'a')
 
         self.data_collection.append(hypercube2)
@@ -445,8 +462,8 @@ class TestImageViewer(object):
         self.viewer.add_data(self.image2)
 
         assert self.viewer.state.reference_data is self.image1
-        assert self.viewer.state.x_att_world is self.image1.world_component_ids[-1]
-        assert self.viewer.state.y_att_world is self.image1.world_component_ids[-2]
+        assert self.viewer.state.x_att_world is self.image1.pixel_component_ids[-1]
+        assert self.viewer.state.y_att_world is self.image1.pixel_component_ids[-2]
         assert self.viewer.state.x_att is self.image1.pixel_component_ids[-1]
         assert self.viewer.state.y_att is self.image1.pixel_component_ids[-2]
 
@@ -461,8 +478,8 @@ class TestImageViewer(object):
         self.viewer.state.reference_data = self.image1
 
         assert self.viewer.state.reference_data is self.image1
-        assert self.viewer.state.x_att_world is self.image1.world_component_ids[-1]
-        assert self.viewer.state.y_att_world is self.image1.world_component_ids[-2]
+        assert self.viewer.state.x_att_world is self.image1.pixel_component_ids[-1]
+        assert self.viewer.state.y_att_world is self.image1.pixel_component_ids[-2]
         assert self.viewer.state.x_att is self.image1.pixel_component_ids[-1]
         assert self.viewer.state.y_att is self.image1.pixel_component_ids[-2]
 
@@ -492,24 +509,36 @@ class TestImageViewer(object):
         self.viewer.add_data(second)
 
         assert self.viewer.state.reference_data is first
-        assert self.viewer.state.x_att_world is first.world_component_ids[-1]
-        assert self.viewer.state.y_att_world is first.world_component_ids[-2]
+        if wcs:
+            assert self.viewer.state.x_att_world is first.world_component_ids[-1]
+            assert self.viewer.state.y_att_world is first.world_component_ids[-2]
+        else:
+            assert self.viewer.state.x_att_world is first.pixel_component_ids[-1]
+            assert self.viewer.state.y_att_world is first.pixel_component_ids[-2]
         assert self.viewer.state.x_att is first.pixel_component_ids[-1]
         assert self.viewer.state.y_att is first.pixel_component_ids[-2]
 
         self.viewer.state.reference_data = second
 
         assert self.viewer.state.reference_data is second
-        assert self.viewer.state.x_att_world is second.world_component_ids[-1]
-        assert self.viewer.state.y_att_world is second.world_component_ids[-2]
+        if wcs:
+            assert self.viewer.state.x_att_world is second.world_component_ids[-1]
+            assert self.viewer.state.y_att_world is second.world_component_ids[-2]
+        else:
+            assert self.viewer.state.x_att_world is second.pixel_component_ids[-1]
+            assert self.viewer.state.y_att_world is second.pixel_component_ids[-2]
         assert self.viewer.state.x_att is second.pixel_component_ids[-1]
         assert self.viewer.state.y_att is second.pixel_component_ids[-2]
 
         self.viewer.state.reference_data = first
 
         assert self.viewer.state.reference_data is first
-        assert self.viewer.state.x_att_world is first.world_component_ids[-1]
-        assert self.viewer.state.y_att_world is first.world_component_ids[-2]
+        if wcs:
+            assert self.viewer.state.x_att_world is first.world_component_ids[-1]
+            assert self.viewer.state.y_att_world is first.world_component_ids[-2]
+        else:
+            assert self.viewer.state.x_att_world is first.pixel_component_ids[-1]
+            assert self.viewer.state.y_att_world is first.pixel_component_ids[-2]
         assert self.viewer.state.x_att is first.pixel_component_ids[-1]
         assert self.viewer.state.y_att is first.pixel_component_ids[-2]
 
@@ -577,8 +606,8 @@ class TestImageViewer(object):
         self.viewer.add_data(self.catalog)
         self.catalog.add_component([4, 5, 6], 'e')
 
-        link1 = LinkSame(self.catalog.id['c'], self.image1.world_component_ids[0])
-        link2 = LinkSame(self.catalog.id['d'], self.image1.world_component_ids[1])
+        link1 = LinkSame(self.catalog.id['c'], self.image1.pixel_component_ids[0])
+        link2 = LinkSame(self.catalog.id['d'], self.image1.pixel_component_ids[1])
         self.data_collection.add_link(link1)
         self.data_collection.add_link(link2)
 
@@ -622,8 +651,8 @@ class TestImageViewer(object):
         assert not self.viewer.layers[2].enabled  # image subset
         assert not self.viewer.layers[3].enabled  # scatter subset
 
-        link1 = LinkSame(self.catalog.id['c'], self.image1.world_component_ids[0])
-        link2 = LinkSame(self.catalog.id['d'], self.image1.world_component_ids[1])
+        link1 = LinkSame(self.catalog.id['c'], self.image1.pixel_component_ids[0])
+        link2 = LinkSame(self.catalog.id['d'], self.image1.pixel_component_ids[1])
         self.data_collection.add_link(link1)
         self.data_collection.add_link(link2)
 
@@ -692,6 +721,25 @@ class TestImageViewer(object):
         self.viewer.toolbar.active_tool = self.viewer.toolbar.tools['mpl:zoom']
         self.viewer.close(warn=False)
 
+    def test_legend(self):
+        from matplotlib.colors import to_hex
+
+        viewer_state = self.viewer.state
+        self.viewer.add_data(self.image1)
+        self.viewer.state.legend.visible = True
+
+        handles, labels, handler_dict = self.viewer.get_handles_legend()
+        assert len(handles) == 1
+        assert labels[0] == 'image1'
+
+        self.data_collection.new_subset_group('test', self.image1.id['x'] > 1)
+        assert len(viewer_state.layers) == 2
+        handles, labels, handler_dict = self.viewer.get_handles_legend()
+        assert len(handles) == 2
+        assert labels[1] == 'test'
+
+        assert to_hex(handles[1].get_facecolor()) == viewer_state.layers[1].color
+
 
 class TestSessions(object):
 
@@ -853,3 +901,45 @@ class TestSessions(object):
         assert layer_state.color == 'b'
 
         ga.close()
+
+
+def test_indexed_data(capsys):
+
+    # Make sure that the image viewer works properly with IndexedData objects
+
+    data_4d = Data(label='hypercube_wcs',
+                   x=np.random.random((3, 5, 4, 3)),
+                   coords=WCS(naxis=4))
+
+    data_2d = IndexedData(data_4d, (2, None, 3, None))
+
+    application = GlueApplication()
+
+    session = application.session
+
+    hub = session.hub
+
+    data_collection = session.data_collection
+    data_collection.append(data_4d)
+    data_collection.append(data_2d)
+
+    viewer = application.new_data_viewer(ImageViewer)
+    viewer.add_data(data_2d)
+
+    assert viewer.state.x_att is data_2d.pixel_component_ids[1]
+    assert viewer.state.y_att is data_2d.pixel_component_ids[0]
+    assert viewer.state.x_att_world is data_2d.world_component_ids[1]
+    assert viewer.state.y_att_world is data_2d.world_component_ids[0]
+
+    process_events()
+
+    application.close()
+
+    # Some exceptions used to happen during callbacks, and these show up
+    # in stderr but don't interrupt the code, so we make sure here that
+    # nothing was printed to stdout nor stderr.
+
+    out, err = capsys.readouterr()
+
+    assert out.strip() == ""
+    assert err.strip() == ""
