@@ -1,24 +1,22 @@
 # Miscellaneous regression tests for the image viewer
 
-from __future__ import absolute_import, division, print_function
-
 import pytest
 import numpy as np
 
 from glue.core import Data
 from glue.viewers.image.qt import ImageViewer
 from glue.core.tests.util import simple_session
-from glue.tests.helpers import requires_matplotlib_ge_14
+from glue.tests.helpers import PYSIDE2_INSTALLED  # noqa
 
 
-@requires_matplotlib_ge_14
+@pytest.mark.skipif('PYSIDE2_INSTALLED')
 @pytest.mark.mpl_image_compare(tolerance=1, savefig_kwargs={'dpi': 50})
 def test_resample_on_zoom():
 
     # For images where the aspect ratio of pixels is fixed to be square, when
     # the user zooms in, the limits of the axes are actually changed twice by
     # matplotlib - a second time when the aspect ratio is enforced. So we need
-    # to make sure that we update the modest_image when this is the case.
+    # to make sure that we update the FRB artist when this is the case.
 
     session = simple_session()
 
@@ -42,4 +40,8 @@ def test_resample_on_zoom():
     image.axes.figure.canvas.motion_notify_event(400 * device_ratio, 210 * device_ratio)
     image.axes.figure.canvas.button_release_event(400 * device_ratio, 210 * device_ratio, 1)
 
-    return image.axes.figure
+    figure = image.axes.figure
+
+    image.close()
+
+    return figure
