@@ -1,9 +1,7 @@
-from __future__ import absolute_import, division, print_function
-
 import uuid
 from collections import defaultdict
 
-from glue.core import BaseData, Coordinates
+from glue.core import BaseData
 from glue.config import colormaps
 from glue.viewers.matplotlib.state import (MatplotlibDataViewerState,
                                            MatplotlibLayerState,
@@ -11,7 +9,7 @@ from glue.viewers.matplotlib.state import (MatplotlibDataViewerState,
                                            DeferredDrawSelectionCallbackProperty as DDSCProperty)
 from glue.core.state_objects import StateAttributeLimitsHelper
 from glue.utils import defer_draw, view_shape
-from glue.external.echo import delay_callback
+from echo import delay_callback
 from glue.core.data_combo_helper import ManualDataComboHelper, ComponentIDComboHelper
 from glue.core.exceptions import IncompatibleDataException
 
@@ -89,9 +87,9 @@ class ImageViewerState(MatplotlibDataViewerState):
                                             'which the images are shown')
     slices = DDCProperty(docstring='The current slice along all dimensions')
     color_mode = DDSCProperty(0, docstring='Whether each layer can have '
-                                                    'its own colormap (``Colormaps``) or '
-                                                    'whether each layer is assigned '
-                                                    'a single color (``One color per layer``)')
+                                           'its own colormap (``Colormaps``) or '
+                                           'whether each layer is assigned '
+                                           'a single color (``One color per layer``)')
 
     dpi = DDCProperty(72, docstring='The resolution (in dots per inch) of density maps, if present')
 
@@ -108,10 +106,10 @@ class ImageViewerState(MatplotlibDataViewerState):
         self.ref_data_helper = ManualDataComboHelper(self, 'reference_data')
 
         self.xw_att_helper = ComponentIDComboHelper(self, 'x_att_world',
-                                                    numeric=False, categorical=False)
+                                                    numeric=False, datetime=False, categorical=False)
 
         self.yw_att_helper = ComponentIDComboHelper(self, 'y_att_world',
-                                                    numeric=False, categorical=False)
+                                                    numeric=False, datetime=False, categorical=False)
 
         self.add_callback('reference_data', self._reference_data_changed, priority=1000)
         self.add_callback('layers', self._layers_changed, priority=1000)
@@ -149,7 +147,7 @@ class ImageViewerState(MatplotlibDataViewerState):
 
     @property
     def _display_world(self):
-        return isinstance(getattr(self.reference_data, 'coords', None), Coordinates)
+        return getattr(self.reference_data, 'coords', None) is not None
 
     def _reference_data_changed(self, *args):
         # This signal can get emitted if just the choices but not the actual
@@ -175,6 +173,11 @@ class ImageViewerState(MatplotlibDataViewerState):
                         self.yw_att_helper.world_coord = False
                     self._update_combo_att()
                     self._set_default_slices()
+                    # We need to make sure that we update x_att and y_att
+                    # at the same time before any other callbacks get called,
+                    # so we do this here manually.
+                    self._on_xatt_world_change()
+                    self._on_yatt_world_change()
 
     def _layers_changed(self, *args):
 
@@ -259,7 +262,10 @@ class ImageViewerState(MatplotlibDataViewerState):
             with delay_callback(self, 'y_att_world', 'x_att'):
 
                 if self.x_att_world == self.y_att_world:
-                    world_ids = self.reference_data.world_component_ids
+                    if self._display_world:
+                        world_ids = self.reference_data.world_component_ids
+                    else:
+                        world_ids = self.reference_data.pixel_component_ids
                     if self.x_att_world == world_ids[-1]:
                         self.y_att_world = world_ids[-2]
                     else:
@@ -279,7 +285,10 @@ class ImageViewerState(MatplotlibDataViewerState):
             with delay_callback(self, 'x_att_world', 'y_att'):
 
                 if self.y_att_world == self.x_att_world:
-                    world_ids = self.reference_data.world_component_ids
+                    if self._display_world:
+                        world_ids = self.reference_data.world_component_ids
+                    else:
+                        world_ids = self.reference_data.pixel_component_ids
                     if self.y_att_world == world_ids[-1]:
                         self.x_att_world = world_ids[-2]
                     else:
@@ -489,9 +498,9 @@ class ImageLayerState(BaseImageLayerState):
     stretch = DDSCProperty(docstring='The stretch used to render the layer, '
                                      'which should be one of ``linear``, '
                                      '``sqrt``, ``log``, or ``arcsinh``')
-    global_sync = DDCProperty(True, docstring='Whether the color and transparency '
-                                              'should be synced with the global '
-                                              'color and transparency for the data')
+    global_sync = DDCProperty(False, docstring='Whether the color and transparency '
+                                               'should be synced with the global '
+                                               'color and transparency for the data')
 
     def __init__(self, layer=None, viewer_state=None, **kwargs):
 
@@ -535,7 +544,7 @@ class ImageLayerState(BaseImageLayerState):
         self.update_from_dict(kwargs)
 
         if self.cmap is None:
-            self.cmap = colormaps.members[0][1]
+            self.cmap = self.layer.style.preferred_cmap or colormaps.members[0][1]
 
     def _update_attribute(self, *args):
         if self.layer is not None:
