@@ -1,12 +1,10 @@
-from __future__ import absolute_import, division, print_function
-
 import pytest
 
 import numpy as np
 from numpy.testing import assert_equal
 
 from glue.core import Data, DataCollection
-from glue.core.coordinates import Coordinates
+from glue.core.coordinates import Coordinates, IdentityCoordinates
 from glue.core.link_helpers import LinkSame
 from glue.core.exceptions import IncompatibleDataException, IncompatibleAttribute
 
@@ -15,17 +13,21 @@ from ..state import ImageViewerState, ImageLayerState, AggregateSlice
 
 class SimpleCoordinates(Coordinates):
 
-    def world2pixel(self, *world):
-        return tuple([0.4 * w for w in world])
+    def __init__(self):
+        super().__init__(pixel_n_dim=3, world_n_dim=3)
 
-    def pixel2world(self, *pixel):
-        return tuple([2.5 * p for p in pixel])
+    def pixel_to_world_values(self, *args):
+        return tuple([2.5 * p for p in args])
 
-    def dependent_axes(self, axis):
-        if axis in (1, 2):
-            return (1, 2)
-        else:
-            return (axis,)
+    def world_to_pixel_values(self, *args):
+        return tuple([0.4 * w for w in args])
+
+    @property
+    def axis_correlation_matrix(self):
+        matrix = np.zeros((self.world_n_dim, self.pixel_n_dim), dtype=bool)
+        matrix[2, 2] = True
+        matrix[0:2, 0:2] = True
+        return matrix
 
 
 class TestImageViewerState(object):
@@ -35,7 +37,8 @@ class TestImageViewerState(object):
 
     def test_pixel_world_linking(self):
 
-        data = Data(label='data', x=[[1, 2], [3, 4]], y=[[5, 6], [7, 8]])
+        data = Data(label='data', x=[[1, 2], [3, 4]], y=[[5, 6], [7, 8]],
+                    coords=IdentityCoordinates(n_dim=2))
         layer_state = ImageLayerState(layer=data, viewer_state=self.state)
         self.state.layers.append(layer_state)
 
@@ -143,7 +146,7 @@ class TestReprojection():
         self.array = np.arange(3024).reshape((6, 7, 8, 9))
 
         # The reference dataset. Shape is (6, 7, 8, 9).
-        self.data1 = Data(x=self.array)
+        self.data1 = Data(x=self.array, coords=IdentityCoordinates(n_dim=4))
         self.data_collection.append(self.data1)
 
         # A dataset with the same shape but not linked. Shape is (6, 7, 8, 9).
@@ -330,3 +333,40 @@ class TestReprojection():
 
         with pytest.raises(IncompatibleAttribute):
             self.viewer_state.layers[0].get_sliced_data()
+
+
+def test_update_x_att_and_y_att():
+
+    # Regression test for a bug that caused y_att to not be updated before
+    # events were sent out about x_att changing.
+
+    viewer_state = ImageViewerState()
+
+    data1 = Data(x=np.ones((3, 4, 5)))
+
+    layer_state1 = ImageLayerState(layer=data1, viewer_state=viewer_state)
+    viewer_state.layers.append(layer_state1)
+
+    data2 = Data(x=np.ones((3, 4, 5)))
+
+    layer_state2 = ImageLayerState(layer=data2, viewer_state=viewer_state)
+    viewer_state.layers.append(layer_state2)
+
+    def check_consistency(*args, **kwargs):
+        # Make sure that x_att and y_att are always for same dataset
+        assert viewer_state.x_att.parent is viewer_state.y_att.parent
+
+    viewer_state.add_global_callback(check_consistency)
+    viewer_state.add_callback('x_att', check_consistency)
+    viewer_state.add_callback('y_att', check_consistency)
+    viewer_state.add_callback('x_att_world', check_consistency)
+    viewer_state.add_callback('y_att_world', check_consistency)
+    viewer_state.add_callback('slices', check_consistency)
+
+    viewer_state.reference_data = data1
+    assert viewer_state.x_att is data1.pixel_component_ids[2]
+    assert viewer_state.y_att is data1.pixel_component_ids[1]
+
+    viewer_state.reference_data = data2
+    assert viewer_state.x_att is data2.pixel_component_ids[2]
+    assert viewer_state.y_att is data2.pixel_component_ids[1]
