@@ -1,9 +1,10 @@
-from __future__ import absolute_import, division, print_function
+from echo import CallbackProperty, SelectionCallbackProperty, keep_in_sync, delay_callback
 
-from glue.external.echo import CallbackProperty, SelectionCallbackProperty, keep_in_sync, delay_callback
+from matplotlib.colors import to_rgba
 
 from glue.core.message import LayerArtistUpdatedMessage
 
+from glue.core.state_objects import State
 from glue.viewers.common.state import ViewerState, LayerState
 
 from glue.utils import defer_draw, avoid_circular
@@ -37,6 +38,69 @@ class DeferredDrawSelectionCallbackProperty(SelectionCallbackProperty):
 VALID_WEIGHTS = ['light', 'normal', 'medium', 'semibold', 'bold', 'heavy', 'black']
 
 
+VALID_LOCATIONS = ['draggable', 'best',
+                   'upper right', 'upper left',
+                   'lower left', 'lower right',
+                   'center left', 'center right',
+                   'lower center', 'upper center']
+
+
+class MatplotlibLegendState(State):
+    """The legend state"""
+
+    visible = DeferredDrawCallbackProperty(False, docstring="Whether to show the legend")
+
+    location = DeferredDrawSelectionCallbackProperty(0, docstring="The location of the legend in the axis")
+
+    title = DeferredDrawCallbackProperty("", docstring='The title of the legend')
+    fontsize = DeferredDrawCallbackProperty(10, docstring='The font size of the title')
+
+    alpha = DeferredDrawCallbackProperty(0.6, docstring='Transparency of the legend frame')
+    frame_color = DeferredDrawCallbackProperty("#ffffff", docstring='Frame color of the legend')
+    show_edge = DeferredDrawCallbackProperty(True, docstring="Whether to show the edge of the frame ")
+    text_color = DeferredDrawCallbackProperty("#000000", docstring='Text color of the legend')
+
+    def __init__(self, *args, **kwargs):
+        MatplotlibLegendState.location.set_choices(self, VALID_LOCATIONS)
+
+        super().__init__(*args, **kwargs)
+        self._set_color_choices()
+
+    def _set_color_choices(self):
+        from glue.config import settings
+
+        self.frame_color = settings.BACKGROUND_COLOR
+        self.text_color = settings.FOREGROUND_COLOR
+
+    @property
+    def edge_color(self):
+        if self.show_edge:
+            return to_rgba(self.text_color, self.alpha)
+        else:
+            return None
+
+    @property
+    def draggable(self):
+        return self.location == 'draggable'
+
+    @property
+    def mpl_location(self):
+        if self.location == 'draggable':
+            return 'best'
+        else:
+            return self.location
+
+    def update_axes_settings_from(self, state):
+        self.visible = state.show_legend
+        self.loc_and_drag = state.loc_and_drag
+        self.alpha = state.alpha
+        self.title = state.title
+        self.fontsize = state.fontsize
+        self.frame_color = state.frame_color
+        self.show_edge = state.show_edge
+        self.text_color = state.text_color
+
+
 class MatplotlibDataViewerState(ViewerState):
     """
     A base class that includes common attributes for viewers based on
@@ -76,6 +140,7 @@ class MatplotlibDataViewerState(ViewerState):
         MatplotlibDataViewerState.y_axislabel_weight.set_choices(self, VALID_WEIGHTS)
 
         super(MatplotlibDataViewerState, self).__init__(*args, **kwargs)
+        self.legend = MatplotlibLegendState(*args, **kwargs)
 
         self.add_callback('aspect', self._adjust_limits_aspect, priority=10000)
         self.add_callback('x_min', self._adjust_limits_aspect_x, priority=10000)
@@ -170,12 +235,15 @@ class MatplotlibDataViewerState(ViewerState):
                 self.y_max = y_max
 
     def update_axes_settings_from(self, state):
+        # axis
         self.x_axislabel_size = state.x_axislabel_size
         self.y_axislabel_size = state.y_axislabel_size
         self.x_axislabel_weight = state.x_axislabel_weight
         self.y_axislabel_weight = state.y_axislabel_weight
         self.x_ticklabel_size = state.x_ticklabel_size
         self.y_ticklabel_size = state.y_ticklabel_size
+        # legend
+        self.legend.update_axes_settings_from(state.legend)
 
     @defer_draw
     def _notify_global(self, *args, **kwargs):
