@@ -1,11 +1,12 @@
-from __future__ import absolute_import, division, print_function
+from textwrap import indent
 
 import numpy as np
 
 from matplotlib.patches import Rectangle
+from matplotlib.artist import setp as msetp
 
 from glue.viewers.matplotlib.mpl_axes import update_appearance_from_settings
-from glue.external.echo import delay_callback
+from echo import delay_callback
 from glue.utils import mpl_to_datetime64
 
 __all__ = ['MatplotlibViewerMixin']
@@ -13,17 +14,24 @@ __all__ = ['MatplotlibViewerMixin']
 SCRIPT_HEADER = """
 # Initialize figure
 fig = plt.figure()
-ax = fig.add_subplot(1, 1, 1, aspect='{aspect}')
+ax = fig.add_subplot(1, 1, 1, aspect='{aspect}', projection={projection})
+
+# for the legend
+legend_handles = []
+legend_labels = []
+legend_handler_dict = dict()
 """.strip()
 
-SCRIPT_FOOTER = """
+LIMIT_SCRIPT = """
 # Set limits
-ax.set_xlim({x_min}, {x_max})
-ax.set_ylim({y_min}, {y_max})
+ax.set_xlim(left={x_min}, right={x_max})
+ax.set_ylim(bottom={y_min}, top={y_max})
+"""
 
-# Set scale (log or linear)
-ax.set_xscale('{x_log_str}')
-ax.set_yscale('{y_log_str}')
+SCRIPT_FOOTER = """
+{limit_script}
+
+{scale_script}
 
 # Set axis label properties
 ax.set_xlabel('{x_axislabel}', weight='{x_axislabel_weight}', size={x_axislabel_size})
@@ -33,11 +41,30 @@ ax.set_ylabel('{y_axislabel}', weight='{y_axislabel_weight}', size={y_axislabel_
 ax.tick_params('x', labelsize={x_ticklabel_size})
 ax.tick_params('y', labelsize={x_ticklabel_size})
 
+# For manual edition of the plot
+#  - Uncomment the next code line (plt.show)
+#  - Also change the matplotlib backend to qt5Agg
+#  - And comment the "plt.close" line
+# plt.show()
+
 # Save figure
 fig.savefig('glue_plot.png')
 plt.close(fig)
 """.strip()
 
+SCRIPT_LEGEND = """
+ax.legend(legend_handles, legend_labels,
+    handler_map=legend_handler_dict,
+    loc='{location}',            # location
+    framealpha={alpha:.2f},      # opacity of the frame
+    title='{title}',             # title of the legend
+    title_fontsize={fontsize},   # fontsize of the title
+    fontsize={fontsize},          # fontsize of the labels
+    facecolor='{frame_color}',
+    edgecolor={edge_color}
+)
+""".strip()
+
 ZORDER_MAX = 100000
 
 
@@ -93,10 +120,20 @@ class MatplotlibViewerMixin(object):
         self.state.add_callback('x_ticklabel_size', self.update_x_ticklabel)
         self.state.add_callback('y_ticklabel_size', self.update_y_ticklabel)
 
+        self.state.legend.add_callback('visible', self.draw_legend)
+        self.state.legend.add_callback('location', self.draw_legend)
+        self.state.legend.add_callback('alpha', self.update_legend)
+        self.state.legend.add_callback('title', self.draw_legend)
+        self.state.legend.add_callback('fontsize', self.draw_legend)
+        self.state.legend.add_callback('frame_color', self.update_legend)
+        self.state.legend.add_callback('show_edge', self.update_legend)
+        self.state.legend.add_callback('text_color', self.update_legend)
+
         self.update_x_axislabel()
         self.update_y_axislabel()
         self.update_x_ticklabel()
         self.update_y_ticklabel()
+        self.draw_legend()
 
     def _update_computation(self, message=None):
         pass
@@ -123,6 +160,55 @@ class MatplotlibViewerMixin(object):
         self.axes.yaxis.get_offset_text().set_fontsize(self.state.y_ticklabel_size)
         self.redraw()
 
+    def get_handles_legend(self):
+        """Collect the handles and labels from each layer artist."""
+        handles = []
+        labels = []
+        handler_dict = {}
+        for layer_artist in self._layer_artist_container:
+            handle, label, handler = layer_artist.get_handle_legend()
+            if handle is not None:
+                handles.append(handle)
+                labels.append(label)
+                if handler is not None:
+                    handler_dict[handle] = handler
+        return handles, labels, handler_dict
+
+    def _update_legend_visual(self, legend):
+        """Update the legend colors and opacity. No redraw."""
+        msetp(legend.get_title(), color=self.state.legend.text_color)
+        msetp(legend.get_texts(), color=self.state.legend.text_color)
+        msetp(legend.get_frame(),
+              alpha=self.state.legend.alpha,
+              facecolor=self.state.legend.frame_color,
+              edgecolor=self.state.legend.edge_color
+              )
+
+    def update_legend(self, *args):
+        """Update the legend colors and opacity."""
+        legend = self.axes.get_legend()
+        if legend is not None:
+            self._update_legend_visual(legend)
+        self.redraw()
+
+    def draw_legend(self, *args):
+        if self.state.legend.visible:
+            handles, labels, handler_map = self.get_handles_legend()
+            kwargs = dict(loc=self.state.legend.mpl_location,
+                          title=self.state.legend.title,
+                          title_fontsize=self.state.legend.fontsize,
+                          fontsize=self.state.legend.fontsize)
+            if handler_map is not None:
+                kwargs["handler_map"] = handler_map
+            legend = self.axes.legend(handles, labels, **kwargs)
+            self._update_legend_visual(legend)
+            legend.set_draggable(self.state.legend.draggable)
+        else:
+            legend = self.axes.get_legend()
+            if legend is not None:
+                legend.remove()
+        self.redraw()
+
     def redraw(self):
         self.figure.canvas.draw_idle()
 
@@ -219,10 +305,33 @@ class MatplotlibViewerMixin(object):
 
     def _script_header(self):
         state_dict = self.state.as_dict()
-        return ['import matplotlib', "matplotlib.use('Agg')", 'import matplotlib.pyplot as plt'], SCRIPT_HEADER.format(**state_dict)
+        proj = "'" + self.state.plot_mode + "'" if hasattr(self.state, 'plot_mode') else "None"
+        state_dict['projection'] = proj
+        return ['import matplotlib',
+                "matplotlib.use('Agg')",
+                "# matplotlib.use('qt5Agg')",
+                'import matplotlib.pyplot as plt'], SCRIPT_HEADER.format(**state_dict)
 
     def _script_footer(self):
+        mode = 'rectilinear' if not hasattr(self.state, 'plot_mode') else self.state.plot_mode
         state_dict = self.state.as_dict()
-        state_dict['x_log_str'] = 'log' if self.state.x_log else 'linear'
-        state_dict['y_log_str'] = 'log' if self.state.y_log else 'linear'
+        temp_str = ''
+        if mode not in ['aitoff', 'hammer', 'lambert', 'mollweide', 'polar']:
+            x_log_str = 'log' if self.state.x_log else 'linear'
+            temp_str += "ax.set_xscale('" + x_log_str + "')\n"
+        if mode not in ['aitoff', 'hammer', 'lambert', 'mollweide']:
+            y_log_str = 'log' if self.state.y_log else 'linear'
+            temp_str += "ax.set_yscale('" + y_log_str + "')\n"
+        state_dict['scale_script'] = "# Set scale (log or linear)\n" + temp_str if temp_str else ''
+        full_sphere = ['aitoff', 'hammer', 'lambert', 'mollweide']
+        state_dict['limit_script'] = '' if mode in full_sphere else LIMIT_SCRIPT.format(**state_dict)
         return [], SCRIPT_FOOTER.format(**state_dict)
+
+    def _script_legend(self):
+        state_dict = self.state.legend.as_dict()
+        state_dict['location'] = self.state.legend.mpl_location
+        state_dict['edge_color'] = self.state.legend.edge_color
+        legend_str = SCRIPT_LEGEND.format(**state_dict)
+        if not self.state.legend.visible:
+            legend_str = indent(legend_str, "# ")
+        return [], legend_str
