@@ -1,10 +1,8 @@
-from __future__ import absolute_import, division, print_function
-
 import os
 
 from qtpy import QtWidgets
 
-from glue.external.echo.qt import autoconnect_callbacks_to_qt
+from echo.qt import autoconnect_callbacks_to_qt
 from glue.utils.qt import load_ui
 
 
