@@ -1,10 +1,9 @@
-from __future__ import absolute_import, division, print_function
-
 import os
 
 from qtpy import QtWidgets
 
-from glue.external.echo.qt import autoconnect_callbacks_to_qt
+from glue.core.coordinate_helpers import dependent_axes
+from echo.qt import autoconnect_callbacks_to_qt
 from glue.utils.qt import load_ui, fix_tab_widget_fontsize
 
 __all__ = ['ProfileOptionsWidget']
@@ -27,6 +26,8 @@ class ProfileOptionsWidget(QtWidgets.QWidget):
 
         self._connections = autoconnect_callbacks_to_qt(viewer_state, self.ui)
         self._connections_axes = autoconnect_callbacks_to_qt(viewer_state, self.ui.axes_editor.ui)
+        connect_kwargs = {'alpha': dict(value_range=(0, 1))}
+        self._connections_legend = autoconnect_callbacks_to_qt(viewer_state.legend, self.ui.legend_editor.ui, connect_kwargs)
 
         self.viewer_state = viewer_state
 
@@ -44,8 +45,8 @@ class ProfileOptionsWidget(QtWidgets.QWidget):
             self.ui.text_warning.hide()
             return
 
-        world_warning = len(self.viewer_state.reference_data.coords
-                            .dependent_axes(self.viewer_state.x_att_pixel.axis)) > 1
+        world_warning = len(dependent_axes(self.viewer_state.reference_data.coords,
+                                           self.viewer_state.x_att_pixel.axis)) > 1
 
         if world_warning:
             self.ui.text_warning.show()
