@@ -1,7 +1,5 @@
 # pylint: disable=I0011,W0613,W0201,W0212,E1101,E1103
 
-from __future__ import absolute_import, division, print_function
-
 import os
 
 import pytest
@@ -15,6 +13,7 @@ from glue.core.subset import SliceSubsetState
 from glue.app.qt import GlueApplication
 from glue.core.component_link import ComponentLink
 from glue.viewers.matplotlib.qt.tests.test_data_viewer import BaseTestMatplotlibDataViewer
+from glue.core.coordinates import IdentityCoordinates
 from glue.viewers.profile.tests.test_state import SimpleCoordinates
 from glue.core.tests.test_state import clone
 from glue.core.state import GlueUnSerializer
@@ -45,12 +44,16 @@ class TestProfileViewer(object):
         self.data.coords = SimpleCoordinates()
         self.data['x'] = np.arange(24).reshape((3, 4, 2))
 
+        self.data2 = Data(label='d2')
+        self.data2['y'] = np.arange(24).reshape((3, 4, 2))
+
         self.app = GlueApplication()
         self.session = self.app.session
         self.hub = self.session.hub
 
         self.data_collection = self.session.data_collection
         self.data_collection.append(self.data)
+        self.data_collection.append(self.data2)
 
         self.viewer = self.app.new_data_viewer(ProfileViewer)
 
@@ -101,7 +104,8 @@ class TestProfileViewer(object):
 
     def test_enabled_layers(self):
 
-        data2 = Data(label='d1', y=np.arange(24).reshape((3, 4, 2)))
+        data2 = Data(label='d1', y=np.arange(24).reshape((3, 4, 2)),
+                     coords=IdentityCoordinates(n_dim=3))
         self.data_collection.append(data2)
 
         self.viewer.add_data(self.data)
@@ -179,6 +183,34 @@ class TestProfileViewer(object):
         self.viewer.state.x_att = self.data.world_component_ids[2]
         assert self.viewer.options_widget().ui.text_warning.text() != ''
 
+    def test_multiple_data(self, tmpdir):
+
+        # Regression test for issues when multiple datasets are present
+        # and the reference data is not the default one.
+
+        self.viewer.add_data(self.data)
+        self.viewer.add_data(self.data2)
+        assert self.viewer.layers[0].enabled
+        assert not self.viewer.layers[1].enabled
+
+        # Make sure that when changing the reference data, which layer
+        # is enabled changes.
+        self.viewer.state.reference_data = self.data2
+        assert not self.viewer.layers[0].enabled
+        assert self.viewer.layers[1].enabled
+
+        # Make sure that everything works fine after saving/reloading
+        filename = tmpdir.join('test_multiple_data.glu').strpath
+        self.session.application.save_session(filename)
+        with open(filename, 'r') as f:
+            session = f.read()
+        state = GlueUnSerializer.loads(session)
+        ga = state.object('__main__')
+        viewer = ga.viewers[0][0]
+        assert not viewer.layers[0].enabled
+        assert viewer.layers[1].enabled
+        ga.close()
+
     @pytest.mark.parametrize('protocol', [1])
     def test_session_back_compat(self, protocol):
 
