@@ -1,11 +1,9 @@
-from __future__ import absolute_import, division, print_function
-
 from collections import OrderedDict
 
 import numpy as np
 
-from glue.core import Subset, Coordinates
-from glue.external.echo import delay_callback
+from glue.core import Subset
+from echo import delay_callback
 from glue.viewers.matplotlib.state import (MatplotlibDataViewerState,
                                            MatplotlibLayerState,
                                            DeferredDrawCallbackProperty as DDCProperty,
@@ -60,7 +58,7 @@ class ProfileViewerState(MatplotlibDataViewerState):
         self.add_callback('normalize', self._reset_y_limits)
 
         self.x_att_helper = ComponentIDComboHelper(self, 'x_att',
-                                                   numeric=False, categorical=False,
+                                                   numeric=False, datetime=False, categorical=False,
                                                    pixel_coord=True)
 
         ProfileViewerState.function.set_choices(self, list(FUNCTIONS))
@@ -78,8 +76,7 @@ class ProfileViewerState(MatplotlibDataViewerState):
 
     @property
     def _display_world(self):
-        return (isinstance(getattr(self.reference_data, 'coords', None), Coordinates) and
-                type(self.reference_data.coords) != Coordinates)
+        return getattr(self.reference_data, 'coords', None) is not None
 
     @defer_draw
     def _update_att(self, *args):
@@ -140,22 +137,39 @@ class ProfileViewerState(MatplotlibDataViewerState):
     @defer_draw
     def _reference_data_changed(self, *args):
 
+        for layer in self.layers:
+            layer.reset_cache()
+
         # This signal can get emitted if just the choices but not the actual
         # reference data change, so we check here that the reference data has
         # actually changed
         if self.reference_data is not getattr(self, '_last_reference_data', None):
             self._last_reference_data = self.reference_data
 
-            if self.reference_data is None:
-                self.x_att_helper.set_multiple_data([])
-            else:
-                self.x_att_helper.set_multiple_data([self.reference_data])
-                if self._display_world:
-                    self.x_att_helper.world_coord = True
-                    self.x_att = self.reference_data.world_component_ids[0]
+            with delay_callback(self, 'x_att'):
+
+                if self.reference_data is None:
+                    self.x_att_helper.set_multiple_data([])
                 else:
-                    self.x_att_helper.world_coord = False
-                    self.x_att = self.reference_data.pixel_component_ids[0]
+                    self.x_att_helper.set_multiple_data([self.reference_data])
+                    if self._display_world:
+                        self.x_att_helper.world_coord = True
+                        self.x_att = self.reference_data.world_component_ids[0]
+                    else:
+                        self.x_att_helper.world_coord = False
+                        self.x_att = self.reference_data.pixel_component_ids[0]
+
+                self._update_att()
+
+    def _update_priority(self, name):
+        if name == 'layers':
+            return 2
+        elif name == 'reference_data':
+            return 1.5
+        elif name.endswith(('_min', '_max')):
+            return 0
+        else:
+            return 1
 
 
 class ProfileLayerState(MatplotlibLayerState):
@@ -230,6 +244,9 @@ class ProfileLayerState(MatplotlibLayerState):
         if self._profile_cache is not None:
             return self._profile_cache
 
+        if not self.visible:
+            return
+
         if not self._viewer_callbacks_set:
             self.viewer_state.add_callback('x_att', self.reset_cache, priority=100000)
             self.viewer_state.add_callback('function', self.reset_cache, priority=100000)
