@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 import warnings
 import numpy as np
 
@@ -11,12 +9,15 @@ from mpl_scatter_density.generic_density_artist import GenericDensityArtist
 from astropy.visualization import (ImageNormalize, LinearStretch, SqrtStretch,
                                    AsinhStretch, LogStretch)
 
-from glue.utils import defer_draw, broadcast_to, nanmax, ensure_numerical
+from glue.utils import defer_draw, broadcast_to, nanmax, ensure_numerical, datetime64_to_mpl
 from glue.viewers.scatter.state import ScatterLayerState
 from glue.viewers.scatter.python_export import python_export_scatter_layer
 from glue.viewers.matplotlib.layer_artist import MatplotlibLayerArtist
 from glue.core.exceptions import IncompatibleAttribute
 
+from matplotlib.lines import Line2D
+
+
 STRETCHES = {'linear': LinearStretch,
              'sqrt': SqrtStretch,
              'arcsinh': AsinhStretch,
@@ -124,6 +125,7 @@ def plot_colored_line(ax, x, y, c=None, cmap=None, vmin=None, vmax=None, **kwarg
     lc = ColoredLineCollection(x, y, **kwargs)
     lc.set_linearcolor(color=c, cmap=cmap, vmin=vmin, vmax=vmax)
     ax.add_collection(lc)
+    return lc
 
 
 class ScatterLayerArtist(MatplotlibLayerArtist):
@@ -141,28 +143,9 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
         self._viewer_state.add_global_callback(self._update_scatter)
         self.state.add_global_callback(self._update_scatter)
 
-        # Scatter
-        self.scatter_artist = self.axes.scatter([], [])
-        self.plot_artist = self.axes.plot([], [], 'o', mec='none')[0]
-        self.errorbar_artist = self.axes.errorbar([], [], fmt='none')
-        self.vector_artist = None
-        self.line_collection = ColoredLineCollection([], [])
-        self.axes.add_collection(self.line_collection)
-
         # Scatter density
         self.density_auto_limits = DensityMapLimits()
-        with warnings.catch_warnings():
-            warnings.filterwarnings("ignore", message='All-NaN slice encountered')
-            self.density_artist = GenericDensityArtist(self.axes, color='white',
-                                                       vmin=self.density_auto_limits.min,
-                                                       vmax=self.density_auto_limits.max,
-                                                       update_while_panning=False,
-                                                       histogram2d_func=self.compute_density_map)
-        self.axes.add_artist(self.density_artist)
-
-        self.mpl_artists = [self.scatter_artist, self.plot_artist,
-                            self.errorbar_artist, self.vector_artist,
-                            self.line_collection, self.density_artist]
+        self._set_axes(axes)
         self.errorbar_index = 2
         self.vector_index = 3
 
@@ -172,6 +155,27 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
         # See also https://github.com/matplotlib/matplotlib/issues/13799
         self._errorbar_keep = None
 
+    def _set_axes(self, axes):
+        self.axes = axes
+        self.scatter_artist = self.axes.scatter([], [])
+        self.plot_artist = self.axes.plot([], [], 'o', mec='none')[0]
+        self.errorbar_artist = self.axes.errorbar([], [], fmt='none')
+        self.vector_artist = None
+        self.line_collection = ColoredLineCollection([], [])
+        self.axes.add_collection(self.line_collection)
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore", message='All-NaN slice encountered')
+            self.density_artist = GenericDensityArtist(self.axes, color='white',
+                                                       vmin=self.density_auto_limits.min,
+                                                       vmax=self.density_auto_limits.max,
+                                                       update_while_panning=False,
+                                                       histogram2d_func=self.compute_density_map,
+                                                       label=None)
+        self.axes.add_artist(self.density_artist)
+        self.mpl_artists = [self.scatter_artist, self.plot_artist,
+                            self.errorbar_artist, self.vector_artist,
+                            self.line_collection, self.density_artist]
+
     def compute_density_map(self, *args, **kwargs):
         try:
             density_map = self.state.compute_density_map(*args, **kwargs)
@@ -193,6 +197,9 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
         try:
             if not self.state.density_map:
                 x = ensure_numerical(self.layer[self._viewer_state.x_att].ravel())
+                if x.dtype.kind == 'M':
+                    x = datetime64_to_mpl(x)
+
         except (IncompatibleAttribute, IndexError):
             # The following includes a call to self.clear()
             self.disable_invalid_attributes(self._viewer_state.x_att)
@@ -203,6 +210,8 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
         try:
             if not self.state.density_map:
                 y = ensure_numerical(self.layer[self._viewer_state.y_att].ravel())
+                if y.dtype.kind == 'M':
+                    y = datetime64_to_mpl(y)
         except (IncompatibleAttribute, IndexError):
             # The following includes a call to self.clear()
             self.disable_invalid_attributes(self._viewer_state.y_att)
@@ -211,6 +220,7 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
             self.enable()
 
         if self.state.markers_visible:
+
             if self.state.density_map:
                 # We don't use x, y here because we actually make use of the
                 # ability of the density artist to call a custom histogram
@@ -219,13 +229,12 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
                 self.plot_artist.set_data([], [])
                 self.scatter_artist.set_offsets(np.zeros((0, 2)))
             else:
-                if self.state.cmap_mode == 'Fixed' and self.state.size_mode == 'Fixed':
+                self.density_artist.set_label(None)
+                if self._use_plot_artist():
                     # In this case we use Matplotlib's plot function because it has much
                     # better performance than scatter.
                     self.plot_artist.set_data(x, y)
-                    self.scatter_artist.set_offsets(np.zeros((0, 2)))
                 else:
-                    self.plot_artist.set_data([], [])
                     offsets = np.vstack((x, y)).transpose()
                     self.scatter_artist.set_offsets(offsets)
         else:
@@ -288,7 +297,8 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
                                                   pivot=self.state.vector_origin,
                                                   headwidth=hw, headlength=hl,
                                                   scale_units='width', angles='xy',
-                                                  scale=10 / self.state.vector_scaling * vmax)
+                                                  scale=10 / self.state.vector_scaling * vmax
+                                                  )
             self.mpl_artists[self.vector_index] = self.vector_artist
 
         if self.state.xerr_visible or self.state.yerr_visible:
@@ -315,7 +325,8 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
             self._errorbar_keep = keep
 
             self.errorbar_artist = self.axes.errorbar(x[keep], y[keep], fmt='none',
-                                                      xerr=xerr, yerr=yerr)
+                                                      xerr=xerr, yerr=yerr
+                                                      )
             self.mpl_artists[self.errorbar_index] = self.errorbar_artist
 
     @defer_draw
@@ -349,7 +360,7 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
 
             else:
 
-                if self.state.cmap_mode == 'Fixed' and self.state.size_mode == 'Fixed':
+                if self._use_plot_artist():
 
                     if force or 'color' in changed or 'fill' in changed:
                         if self.state.fill:
@@ -483,7 +494,10 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
                                        self.state.markers_visible)
                 else:
                     artist.set_visible(self.state.visible)
-
+        if self._use_plot_artist():
+            self.scatter_artist.set_visible(False)
+        else:
+            self.plot_artist.set_visible(False)
         self.redraw()
 
     @defer_draw
@@ -519,3 +533,45 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
         # Clean up the density artist to avoid circular references to do a
         # reference to the self.histogram2d method in density artist.
         self.density_artist = None
+
+    def get_handle_legend(self):
+        if self.enabled and self.state.visible:
+            handles = []
+            if self.state.markers_visible:
+                if self.state.density_map:
+                    if self.state.cmap_mode == 'Fixed':
+                        color = self.get_layer_color()
+                    else:
+                        color = self.layer.style.color
+                    handle = Line2D([0, ], [0, ], marker=".", linestyle="none",
+                                    ms=self.state.size, alpha=self.state.alpha,
+                                    color=color)
+                    handles.append(handle)  # as placeholder
+                else:
+                    if self._use_plot_artist():
+                        handles.append(self.plot_artist)
+                    else:
+                        handles.append(self.scatter_artist)
+
+            if self.state.line_visible:
+                handles.append(self.line_collection)
+
+            if self.state.vector_visible:
+                handles.append(self.vector_artist)
+
+            if self.state.xerr_visible or self.state.yerr_visible:
+                handles.append(self.errorbar_artist)
+
+            handles = tuple(handles)
+            if len(handles) > 0:
+                return handles, self.layer.label, None
+            else:
+                return None, None, None
+
+        else:
+            return None, None, None
+
+    def _use_plot_artist(self):
+        res = self.state.cmap_mode == 'Fixed' and self.state.size_mode == 'Fixed'
+        return res and (not hasattr(self._viewer_state, 'plot_mode') or
+                        not self._viewer_state.plot_mode == 'polar')
