@@ -1,7 +1,5 @@
 # pylint: disable=I0011,W0613,W0201,W0212,E1101,E1103
 
-from __future__ import absolute_import, division, print_function
-
 import os
 from collections import Counter
 
@@ -14,6 +12,7 @@ from glue.config import colormaps
 from glue.core.message import SubsetUpdateMessage
 from glue.core import HubListener, Data
 from glue.core.roi import XRangeROI, RectangularROI, CircularROI
+from glue.core.roi_pretransforms import ProjectionMplTransform
 from glue.core.subset import RoiSubsetState, AndState
 from glue import core
 from glue.core.component_id import ComponentID
@@ -27,6 +26,8 @@ from ..data_viewer import ScatterViewer
 
 DATA = os.path.join(os.path.dirname(__file__), 'data')
 
+fullsphere_projections = ['aitoff', 'hammer', 'lambert', 'mollweide']
+
 
 class TestScatterCommon(BaseTestMatplotlibDataViewer):
     def init_data(self):
@@ -66,8 +67,8 @@ class TestScatterViewer(object):
         # Check defaults when we add data
         self.viewer.add_data(self.data)
 
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]:World 0'
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]:World 0'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]'
 
         assert viewer_state.x_att is self.data.id['x']
         assert_allclose(viewer_state.x_min, -1.1 - 0.18)
@@ -121,8 +122,8 @@ class TestScatterViewer(object):
 
     def test_remove_data(self):
         self.viewer.add_data(self.data)
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]:World 0'
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]:World 0'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]'
         self.data_collection.remove(self.data)
         assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == ''
         assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == ''
@@ -138,8 +139,8 @@ class TestScatterViewer(object):
         self.data.add_component([3, 4, 1, 2], 'a')
         assert self.viewer.state.x_att is self.data.id['x']
         assert self.viewer.state.y_att is self.data.id['y']
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:z:a:Coordinate components:Pixel Axis 0 [x]:World 0'
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == 'Main components:x:y:z:a:Coordinate components:Pixel Axis 0 [x]:World 0'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:z:a:Coordinate components:Pixel Axis 0 [x]'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == 'Main components:x:y:z:a:Coordinate components:Pixel Axis 0 [x]'
 
     def test_nonnumeric_first_component(self):
         # regression test for #208. Shouldn't complain if
@@ -231,7 +232,7 @@ class TestScatterViewer(object):
         test = ComponentID('test')
         self.data.update_id(self.viewer.state.x_att, test)
         assert self.viewer.state.x_att is test
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:test:y:z:Coordinate components:Pixel Axis 0 [x]:World 0'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:test:y:z:Coordinate components:Pixel Axis 0 [x]'
 
     def test_nan_component(self):
         # regression test for case when all values are NaN in a component
@@ -599,6 +600,41 @@ class TestScatterViewer(object):
 
         ga.close()
 
+    def test_datetime64_disabled(self, capsys):
+
+        # Make sure that datetime components aren't options for the vector and
+        # error markers.
+
+        data = Data(label='test')
+        data.add_component(np.array([100, 200, 300, 400], dtype='M8[D]'), 't1')
+        data.add_component(np.array([200, 300, 400, 500], dtype='M8[D]'), 't2')
+        data.add_component(np.array([200., 300., 400., 500.]), 'x')
+        data.add_component(np.array([200., 300., 400., 500.]), 'y')
+        self.data_collection.append(data)
+
+        self.viewer.add_data(data)
+        self.viewer.state.x_att = data.id['x']
+        self.viewer.state.y_att = data.id['y']
+        self.viewer.state.layers[0].cmap_mode = 'Linear'
+        self.viewer.state.layers[0].cmap_att = data.id['x']
+        self.viewer.state.layers[0].size_mode = 'Linear'
+        self.viewer.state.layers[0].size_att = data.id['y']
+        self.viewer.state.layers[0].vector_visible = True
+        self.viewer.state.layers[0].xerr_visible = True
+        self.viewer.state.layers[0].yerr_visible = True
+
+        process_events()
+
+        self.viewer.state.x_att = data.id['t1']
+        self.viewer.state.y_att = data.id['t2']
+
+        process_events()
+
+        #  We use capsys here because the # error is otherwise only apparent in stderr.
+        out, err = capsys.readouterr()
+        assert out.strip() == ""
+        assert err.strip() == ""
+
     def test_density_map_incompatible_subset(self, capsys):
 
         # Regression test for a bug that caused the scatter viewer to crash
@@ -643,3 +679,294 @@ class TestScatterViewer(object):
         assert not self.viewer.state.layers[0].xerr_visible
         assert not self.viewer.state.layers[0].yerr_visible
         assert not self.viewer.state.layers[0].vector_visible
+
+    def test_legend(self):
+        viewer_state = self.viewer.state
+
+        self.viewer.add_data(self.data)
+        viewer_state.legend.visible = True
+
+        handles, labels, handler_dict = self.viewer.get_handles_legend()
+        assert len(handles) == 1
+        assert labels[0] == 'd1'
+
+        self.data_collection.new_subset_group('test', self.data.id['x'] > 1)
+        assert len(viewer_state.layers) == 2
+        handles, labels, handler_dict = self.viewer.get_handles_legend()
+
+        assert len(handles) == 2
+        assert labels[1] == 'test'
+        print(handles[1][0])
+        # assert handles[1][0].get_color() == viewer_state.layers[1].state.color
+
+        # Add a non visible layer
+        data2 = Data(label='d2', x=[3.4, 2.3, -1.1, 0.3], y=[3.2, 3.3, 3.4, 3.5])
+        self.data_collection.append(data2)
+
+        self.viewer.add_data(data2)
+        assert len(viewer_state.layers) == 4
+
+        # 'd2' is not enabled (no linked component)
+        handles, labels, handler_dict = self.viewer.get_handles_legend()
+        assert len(handles) == 2
+
+    def test_changing_plot_modes(self):
+        viewer_state = self.viewer.state
+        viewer_state.plot_mode = 'polar'
+        assert 'polar' in str(type(self.viewer.axes)).lower()
+        viewer_state.plot_mode = 'aitoff'
+        assert 'aitoff' in str(type(self.viewer.axes)).lower()
+        viewer_state.plot_mode = 'hammer'
+        assert 'hammer' in str(type(self.viewer.axes)).lower()
+        viewer_state.plot_mode = 'lambert'
+        assert 'lambert' in str(type(self.viewer.axes)).lower()
+        viewer_state.plot_mode = 'mollweide'
+        assert 'mollweide' in str(type(self.viewer.axes)).lower()
+
+    def test_limit_log_set_polar(self):
+        self.viewer.add_data(self.data)
+        viewer_state = self.viewer.state
+        viewer_state.plot_mode = "polar"
+        axes = self.viewer.axes
+
+        viewer_state.x_min = 0.5
+        viewer_state.x_max = 1.5
+        assert_allclose(axes.get_xlim(), [0.5, 1.5])
+
+        viewer_state.y_min = -2.5
+        viewer_state.y_max = 2.5
+        assert_allclose(axes.get_ylim(), [-2.5, 2.5])
+
+        viewer_state.y_log = True
+        assert axes.get_yscale() == 'log'
+
+    def test_limit_set_fullsphere(self):
+        # Make sure that the full-sphere projections ignore instead of throwing exceptions
+        self.viewer.add_data(self.data)
+        viewer_state = self.viewer.state
+
+        for proj in fullsphere_projections:
+            viewer_state.plot_mode = proj
+            error_msg = 'Issue with {} projection'.format(proj)
+            axes = self.viewer.axes
+            viewer_state.x_min = 0.5
+            viewer_state.x_max = 1.5
+            viewer_state.y_min = -2.5
+            viewer_state.y_max = 2.5
+            assert_allclose(axes.get_xlim(), [-np.pi, np.pi], err_msg=error_msg)
+            assert_allclose(axes.get_ylim(), [-np.pi / 2, np.pi / 2], err_msg=error_msg)
+
+    def test_changing_mode_limits(self):
+        self.viewer.add_data(self.data)
+        viewer_state = self.viewer.state
+        old_xmin = viewer_state.x_min
+        old_xmax = viewer_state.x_max
+        old_ymin = viewer_state.y_min
+        old_ymax = viewer_state.y_max
+        # Make sure limits are reset first
+        viewer_state.x_max += 3
+
+        viewer_state.plot_mode = 'polar'
+        assert_allclose(viewer_state.x_min, old_xmin)
+        assert_allclose(viewer_state.x_max, old_xmax)
+        assert_allclose(self.viewer.axes.get_xlim(), [old_xmin, old_xmax])
+        assert_allclose(viewer_state.y_min, old_ymin)
+        assert_allclose(viewer_state.y_max, old_ymax)
+        assert_allclose(self.viewer.axes.get_ylim(), [old_ymin, old_ymax])
+
+        viewer_state.plot_mode = 'rectilinear'
+        assert_allclose(viewer_state.x_min, old_xmin)
+        assert_allclose(viewer_state.x_max, old_xmax)
+        assert_allclose(self.viewer.axes.get_xlim(), [old_xmin, old_xmax])
+        assert_allclose(viewer_state.y_min, old_ymin)
+        assert_allclose(viewer_state.y_max, old_ymax)
+        assert_allclose(self.viewer.axes.get_ylim(), [old_ymin, old_ymax])
+
+        for proj in fullsphere_projections:
+            viewer_state.plot_mode = 'rectilinear'
+            viewer_state.plot_mode = proj
+            error_msg = 'Issue with {} projection'.format(proj)
+            assert_allclose(viewer_state.x_min, -np.pi)
+            assert_allclose(viewer_state.x_max, np.pi)
+            assert_allclose(self.viewer.axes.get_xlim(), [-np.pi, np.pi], err_msg=error_msg)
+            assert_allclose(viewer_state.y_min, -np.pi / 2)
+            assert_allclose(viewer_state.y_max, np.pi / 2)
+            assert_allclose(self.viewer.axes.get_ylim(), [-np.pi / 2, np.pi / 2], err_msg=error_msg)
+
+    def test_changing_mode_log(self):
+        # Test to make sure we reset the log axes to false when changing modes to prevent problems
+        self.viewer.add_data(self.data)
+        viewer_state = self.viewer.state
+        viewer_state.x_log = True
+        viewer_state.y_log = True
+
+        viewer_state.plot_mode = 'polar'
+        assert not viewer_state.x_log
+        assert not viewer_state.y_log
+        assert self.viewer.axes.get_xscale() == 'linear'
+        assert self.viewer.axes.get_yscale() == 'linear'
+        viewer_state.y_log = True
+
+        viewer_state.plot_mode = 'rectilinear'
+        assert not viewer_state.x_log
+        assert not viewer_state.y_log
+        assert self.viewer.axes.get_xscale() == 'linear'
+        assert self.viewer.axes.get_yscale() == 'linear'
+
+        for proj in fullsphere_projections:
+            viewer_state.plot_mode = 'rectilinear'
+            viewer_state.x_log = True
+            viewer_state.y_log = True
+            viewer_state.plot_mode = proj
+            error_msg = 'Issue with {} projection'.format(proj)
+            assert not viewer_state.x_log, error_msg
+            assert not viewer_state.y_log, error_msg
+            assert self.viewer.axes.get_xscale() == 'linear', error_msg
+            assert self.viewer.axes.get_yscale() == 'linear', error_msg
+
+    def test_full_circle_utility(self):
+        # Make sure that the full circle function behaves well
+        self.viewer.add_data(self.data)
+        viewer_state = self.viewer.state
+        old_xmin = viewer_state.x_min
+        old_xmax = viewer_state.x_max
+        old_ymin = viewer_state.y_min
+        old_ymax = viewer_state.y_max
+        viewer_state.full_circle()
+        assert_allclose([viewer_state.x_min, viewer_state.x_max], [old_xmin, old_xmax])
+        assert_allclose([viewer_state.y_min, viewer_state.y_max], [old_ymin, old_ymax])
+
+        viewer_state.plot_mode = 'polar'
+        viewer_state.full_circle()
+        assert_allclose([viewer_state.x_min, viewer_state.x_max], [0, 2 * np.pi])
+        assert_allclose([viewer_state.y_min, viewer_state.y_max], [old_ymin, old_ymax])
+
+        for proj in fullsphere_projections:
+            error_msg = 'Issue with {} projection'.format(proj)
+            viewer_state.plot_mode = proj
+            viewer_state.full_circle()
+            assert_allclose([viewer_state.x_min, viewer_state.x_max], [-np.pi, np.pi], err_msg=error_msg)
+            assert_allclose([viewer_state.y_min, viewer_state.y_max], [-np.pi / 2, np.pi / 2], err_msg=error_msg)
+
+    def test_limits_log_widget_polar_cartesian(self):
+        ui = self.viewer.options_widget().ui
+        viewer_state = self.viewer.state
+        viewer_state.plot_mode = 'polar'
+        assert not ui.bool_x_log.isEnabled()
+        assert not ui.bool_x_log_.isEnabled()
+        assert ui.bool_y_log.isEnabled()
+        assert ui.bool_y_log_.isEnabled()
+        assert ui.valuetext_x_min.isEnabled()
+        assert ui.button_flip_x.isEnabled()
+        assert ui.valuetext_x_max.isEnabled()
+        assert ui.valuetext_y_min.isEnabled()
+        assert ui.button_flip_y.isEnabled()
+        assert ui.valuetext_y_max.isEnabled()
+        assert not ui.button_full_circle.isHidden()
+
+        viewer_state.plot_mode = 'rectilinear'
+        assert ui.bool_x_log.isEnabled()
+        assert ui.bool_x_log_.isEnabled()
+        assert ui.bool_y_log.isEnabled()
+        assert ui.bool_y_log_.isEnabled()
+        assert ui.valuetext_x_min.isEnabled()
+        assert ui.button_flip_x.isEnabled()
+        assert ui.valuetext_x_max.isEnabled()
+        assert ui.valuetext_y_min.isEnabled()
+        assert ui.button_flip_y.isEnabled()
+        assert ui.valuetext_y_max.isEnabled()
+        assert ui.button_full_circle.isHidden()
+        assert ui.button_full_circle.isHidden()
+
+    def test_limits_log_widget_fullsphere(self):
+        ui = self.viewer.options_widget().ui
+        viewer_state = self.viewer.state
+        for proj in fullsphere_projections:
+            error_msg = 'Issue with {} projection'.format(proj)
+            viewer_state.plot_mode = proj
+            not ui.bool_x_log.isEnabled()
+            assert not ui.bool_x_log_.isEnabled(), error_msg
+            assert not ui.bool_y_log.isEnabled(), error_msg
+            assert not ui.bool_y_log_.isEnabled(), error_msg
+            assert not ui.valuetext_x_min.isEnabled(), error_msg
+            assert not ui.button_flip_x.isEnabled(), error_msg
+            assert not ui.valuetext_x_max.isEnabled(), error_msg
+            assert not ui.valuetext_y_min.isEnabled(), error_msg
+            assert not ui.button_flip_y.isEnabled(), error_msg
+            assert not ui.valuetext_y_max.isEnabled(), error_msg
+            assert ui.button_full_circle.isHidden(), error_msg
+
+            viewer_state.plot_mode = 'rectilinear'
+            assert ui.bool_x_log.isEnabled()
+            assert ui.bool_x_log_.isEnabled()
+            assert ui.bool_y_log.isEnabled()
+            assert ui.bool_y_log_.isEnabled()
+            assert ui.valuetext_x_min.isEnabled()
+            assert ui.button_flip_x.isEnabled()
+            assert ui.valuetext_x_max.isEnabled()
+            assert ui.valuetext_y_min.isEnabled()
+            assert ui.button_flip_y.isEnabled()
+            assert ui.valuetext_y_max.isEnabled()
+            assert ui.button_full_circle.isHidden()
+
+    def test_apply_roi_polar(self):
+        self.viewer.add_data(self.data)
+        viewer_state = self.viewer.state
+        roi = RectangularROI(0, 0.5, 0, 0.5)
+        viewer_state.plot_mode = 'polar'
+        viewer_state.full_circle()
+        assert len(self.viewer.layers) == 1
+
+        self.viewer.apply_roi(roi)
+
+        assert len(self.viewer.layers) == 2
+        assert len(self.data.subsets) == 1
+
+        assert_allclose(self.data.subsets[0].to_mask(), [1, 0, 0, 0])
+
+        state = self.data.subsets[0].subset_state
+        assert isinstance(state, RoiSubsetState)
+        assert state.pretransform
+        pretrans = state.pretransform
+        assert isinstance(pretrans, ProjectionMplTransform)
+        assert pretrans._state['projection'] == 'polar'
+        assert_allclose(pretrans._state['x_lim'], [viewer_state.x_min, viewer_state.x_max])
+        assert_allclose(pretrans._state['y_lim'], [viewer_state.y_min, viewer_state.y_max])
+        assert pretrans._state['x_scale'] == 'linear'
+        assert pretrans._state['y_scale'] == 'linear'
+        self.data.subsets[0].delete()
+
+        viewer_state.y_log = True
+        self.viewer.apply_roi(roi)
+        state = self.data.subsets[0].subset_state
+        assert state.pretransform
+        pretrans = state.pretransform
+        assert isinstance(pretrans, ProjectionMplTransform)
+        assert pretrans._state['y_scale'] == 'log'
+
+    def test_apply_roi_fullsphere(self):
+        self.viewer.add_data(self.data)
+        viewer_state = self.viewer.state
+        roi = RectangularROI(0, 0.5, 0, 0.5)
+
+        for proj in fullsphere_projections:
+            viewer_state.plot_mode = proj
+            assert len(self.viewer.layers) == 1
+
+            self.viewer.apply_roi(roi)
+
+            assert len(self.viewer.layers) == 2
+            assert len(self.data.subsets) == 1
+
+            subset = self.data.subsets[0]
+            state = subset.subset_state
+            assert isinstance(state, RoiSubsetState)
+            assert state.pretransform
+            pretrans = state.pretransform
+            assert isinstance(pretrans, ProjectionMplTransform)
+            assert pretrans._state['projection'] == proj
+            assert_allclose(pretrans._state['x_lim'], [viewer_state.x_min, viewer_state.x_max])
+            assert_allclose(pretrans._state['y_lim'], [viewer_state.y_min, viewer_state.y_max])
+            assert pretrans._state['x_scale'] == 'linear'
+            assert pretrans._state['y_scale'] == 'linear'
+            subset.delete()
