@@ -1,7 +1,5 @@
 # -*- coding: utf-8 -*-
 
-from __future__ import absolute_import, division, print_function
-
 import numpy as np
 
 from glue.core import BaseData, Subset
@@ -12,10 +10,12 @@ from glue.viewers.matplotlib.state import (MatplotlibDataViewerState,
                                            DeferredDrawCallbackProperty as DDCProperty,
                                            DeferredDrawSelectionCallbackProperty as DDSCProperty)
 from glue.core.state_objects import StateAttributeLimitsHelper
-from glue.external.echo import keep_in_sync, delay_callback
-from glue.core.data_combo_helper import ComponentIDComboHelper
+from echo import keep_in_sync, delay_callback
+from glue.core.data_combo_helper import ComponentIDComboHelper, ComboHelper
 from glue.core.exceptions import IncompatibleAttribute
 
+from matplotlib.projections import get_projection_names
+
 __all__ = ['ScatterViewerState', 'ScatterLayerState']
 
 
@@ -27,6 +27,7 @@ class ScatterViewerState(MatplotlibDataViewerState):
     x_att = DDSCProperty(docstring='The attribute to show on the x-axis', default_index=0)
     y_att = DDSCProperty(docstring='The attribute to show on the y-axis', default_index=1)
     dpi = DDCProperty(72, docstring='The resolution (in dots per inch) of density maps, if present')
+    plot_mode = DDSCProperty(docstring="Whether to plot the data in cartesian, polar or another projection")
 
     def __init__(self, **kwargs):
 
@@ -49,6 +50,10 @@ class ScatterViewerState(MatplotlibDataViewerState):
         self.x_att_helper = ComponentIDComboHelper(self, 'x_att', pixel_coord=True, world_coord=True)
         self.y_att_helper = ComponentIDComboHelper(self, 'y_att', pixel_coord=True, world_coord=True)
 
+        self.plot_mode_helper = ComboHelper(self, 'plot_mode')
+        self.plot_mode_helper.choices = [proj for proj in get_projection_names() if proj not in ['3d', 'scatter_density']]
+        self.plot_mode_helper.selection = 'rectilinear'
+
         self.update_from_dict(kwargs)
 
         self.add_callback('x_log', self._reset_x_limits)
@@ -82,6 +87,12 @@ class ScatterViewerState(MatplotlibDataViewerState):
         """
         self.y_lim_helper.flip_limits()
 
+    def full_circle(self):
+        if not self.plot_mode == 'polar':
+            return
+        self.x_min = 0
+        self.x_max = 2 * np.pi
+
     @property
     def x_categories(self):
         return self._categories(self.x_att)
@@ -238,22 +249,22 @@ class ScatterLayerState(MatplotlibLayerState):
                                                           limits_cache=self.limits_cache)
 
         self.cmap_att_helper = ComponentIDComboHelper(self, 'cmap_att',
-                                                      numeric=True, categorical=False)
+                                                      numeric=True, datetime=False, categorical=False)
 
         self.size_att_helper = ComponentIDComboHelper(self, 'size_att',
-                                                      numeric=True, categorical=False)
+                                                      numeric=True, datetime=False, categorical=False)
 
         self.xerr_att_helper = ComponentIDComboHelper(self, 'xerr_att',
-                                                      numeric=True, categorical=False)
+                                                      numeric=True, datetime=False, categorical=False)
 
         self.yerr_att_helper = ComponentIDComboHelper(self, 'yerr_att',
-                                                      numeric=True, categorical=False)
+                                                      numeric=True, datetime=False, categorical=False)
 
         self.vx_att_helper = ComponentIDComboHelper(self, 'vx_att',
-                                                    numeric=True, categorical=False)
+                                                    numeric=True, datetime=False, categorical=False)
 
         self.vy_att_helper = ComponentIDComboHelper(self, 'vy_att',
-                                                    numeric=True, categorical=False)
+                                                    numeric=True, datetime=False, categorical=False)
 
         points_mode_display = {'auto': 'Density map or markers (auto)',
                                'markers': 'Markers',
@@ -293,6 +304,11 @@ class ScatterLayerState(MatplotlibLayerState):
         ScatterLayerState.stretch.set_choices(self, ['linear', 'sqrt', 'arcsinh', 'log'])
         ScatterLayerState.stretch.set_display_func(self, stretch_display.get)
 
+        if self.viewer_state is not None:
+            self.viewer_state.add_callback('x_att', self._on_xy_change, priority=10000)
+            self.viewer_state.add_callback('y_att', self._on_xy_change, priority=10000)
+            self._on_xy_change()
+
         self.add_callback('layer', self._on_layer_change)
         if layer is not None:
             self._on_layer_change()
@@ -305,6 +321,34 @@ class ScatterLayerState(MatplotlibLayerState):
 
         self.update_from_dict(kwargs)
 
+    def _on_xy_change(self, *event):
+
+        if self.viewer_state.x_att is None or self.viewer_state.y_att is None:
+            return
+
+        if isinstance(self.layer, BaseData):
+            layer = self.layer
+        else:
+            layer = self.layer.data
+
+        try:
+            x_datetime = layer.get_kind(self.viewer_state.x_att) == 'datetime'
+        except IncompatibleAttribute:
+            x_datetime = False
+
+        try:
+            y_datetime = layer.get_kind(self.viewer_state.y_att) == 'datetime'
+        except IncompatibleAttribute:
+            y_datetime = False
+
+        with delay_callback(self, 'xerr_visible', 'yerr_visible', 'vector_visible'):
+            if x_datetime:
+                self.xerr_visible = False
+            if y_datetime:
+                self.yerr_visible = False
+            if x_datetime or y_datetime:
+                self.vector_visible = False
+
     def _on_layer_change(self, layer=None):
 
         with delay_callback(self, 'cmap_vmin', 'cmap_vmax', 'size_vmin', 'size_vmax', 'density_map'):
